@@ -20,35 +20,37 @@
 
 import traceback
 from collections import deque
-from PyQt4 import QtCore
+from PyQt4 import QtCore, QtNetwork
 from picard.metadata import Metadata, run_album_metadata_processors, run_track_metadata_processors
 from picard.dataobj import DataObject
 from picard.file import File
 from picard.track import Track
 from picard.script import ScriptParser
 from picard.ui.item import Item
-from picard.util import format_time, partial, translate_artist, queue, mbid_validate
+from picard.util import format_time, partial, queue, mbid_validate, asciipunct
 from picard.cluster import Cluster
-from picard.mbxml import release_to_metadata, track_to_metadata, media_formats_from_node
+from picard.mbxml import release_to_metadata, medium_to_metadata, track_to_metadata, media_formats_from_node, label_info_from_node
 from picard.const import VARIOUS_ARTISTS_ID
 
 
 class Album(DataObject, Item):
 
+    release_group_loaded = QtCore.pyqtSignal()
+
     def __init__(self, id, discid=None):
         DataObject.__init__(self, id)
         self.metadata = Metadata()
         self.tracks = []
         self.format_str = ""
-        self.tracks_str = ""
         self.loaded = False
+        self.load_task = None
         self.rgloaded = False
         self.rgid = None
         self._files = 0
         self._requests = 0
+        self._tracks_loaded = False
         self._discid = discid
         self._after_load_callbacks = queue.Queue()
-        self._metadata_plugins = deque()
         self.other_versions = []
         self.unmatched_files = Cluster(_("Unmatched Files"), special=True, related_album=self, hide_if_empty=True)
 
@@ -65,18 +67,22 @@ class Album(DataObject, Item):
 
     def _parse_release(self, document):
         self.log.debug("Loading release %r", self.id)
+        self._tracks_loaded = False
 
         release_node = document.metadata[0].release[0]
         if release_node.id != self.id:
-            album = self.tagger.get_album_by_id(release_node.id)
-            self.tagger.albumids[self.id] = release_node.id
-            self.id = release_node.id
+            self.tagger.mbid_redirects[self.id] = release_node.id
+            album = self.tagger.albums.get(release_node.id)
             if album:
+                self.log.debug("Release %r already loaded", release_node.id)
                 album.match_files(self.unmatched_files.files)
                 album.update()
                 self.tagger.remove_album(self)
-                self.log.debug("Release %r already loaded", self.id)
                 return False
+            else:
+                del self.tagger.albums[self.id]
+                self.tagger.albums[release_node.id] = self
+                self.id = release_node.id
 
         # Get release metadata
         m = self._new_metadata
@@ -88,81 +94,47 @@ class Album(DataObject, Item):
         if self._discid:
             m['musicbrainz_discid'] = self._discid
 
-        # 'Translate' artist name
-        if self.config.setting['translate_artist_names']:
-            m['albumartist'] = m['artist'] = translate_artist(m['artist'], m['artistsort'])
-
         # Custom VA name
-        if m['musicbrainz_artistid'] == VARIOUS_ARTISTS_ID:
-            m['albumartistsort'] = m['artistsort'] = m['albumartist'] = m['artist'] = self.config.setting['va_name']
+        if m['musicbrainz_albumartistid'] == VARIOUS_ARTISTS_ID:
+            m['albumartistsort'] = m['albumartist'] = self.config.setting['va_name']
 
-        ignore_tags = [s.strip() for s in self.config.setting['ignore_tags'].split(',')]
-        first_artist = None
-        compilation = False
-        track_counts = []
+        # Convert Unicode punctuation
+        if self.config.setting['convert_punctuation']:
+            m.apply_func(asciipunct)
 
         m['totaldiscs'] = release_node.medium_list[0].count
 
-        plugins = partial(run_album_metadata_processors, self, m, release_node)
-        self._metadata_plugins.append(plugins)
-
-        for medium in release_node.medium_list[0].medium:
-            discnumber = medium.position[0].text
-            track_list = medium.track_list[0]
-            totaltracks = track_list.count
-            track_counts.append(totaltracks)
-            discsubtitle = medium.title[0].text if "title" in medium.children else ""
-            format = medium.format[0].text if "format" in medium.children else ""
-
-            for node in track_list.track:
-                t = Track(node.recording[0].id, self)
-                self._new_tracks.append(t)
-
-                # Get track metadata
-                tm = t.metadata
-                tm.copy(m)
-                tm['discnumber'] = discnumber
-                tm['discsubtitle'] = discsubtitle
-                tm['totaltracks'] = totaltracks
-                if format: tm['media'] = format
-
-                track_to_metadata(node, config=self.config, track=t)
-                m.length += tm.length
-
-                artist_id = tm['musicbrainz_artistid']
-                if compilation is False:
-                    if first_artist is None:
-                        first_artist = artist_id
-                    if first_artist != artist_id:
-                        compilation = True
-                        for track in self._new_tracks:
-                            track.metadata['compilation'] = '1'
-                else:
-                    tm['compilation'] = '1'
-
-                t._customize_metadata(ignore_tags)
-                plugins = partial(run_track_metadata_processors, self, tm, release_node, node)
-                self._metadata_plugins.append(plugins)
-
-        self.tracks_str = " + ".join(track_counts)
+        # Run album metadata plugins
+        try:
+            run_album_metadata_processors(self, m, release_node)
+        except:
+            self.log.error(traceback.format_exc())
 
+        self._release_node = release_node
         return True
 
     def _release_request_finished(self, document, http, error):
+        if self.load_task is None:
+            return
+        self.load_task = None
         parsed = False
         try:
             if error:
                 self.log.error("%r", unicode(http.errorString()))
                 # Fix for broken NAT releases
-                files = list(self.unmatched_files.files)
-                for file in files:
-                    trackid = file.metadata["musicbrainz_trackid"]
-                    if mbid_validate(trackid) and file.metadata["album"] == self.config.setting["nat_name"]:
-                        self.tagger.move_file_to_nat(file, trackid)
-                        self.tagger.nats.update()
-                if not self.get_num_unmatched_files():
-                    self.tagger.remove_album(self)
-                    error = False
+                if error == QtNetwork.QNetworkReply.ContentNotFoundError:
+                    nats = False
+                    nat_name = self.config.setting["nat_name"]
+                    files = list(self.unmatched_files.files)
+                    for file in files:
+                        trackid = file.metadata["musicbrainz_trackid"]
+                        if mbid_validate(trackid) and file.metadata["album"] == nat_name:
+                            nats = True
+                            self.tagger.move_file_to_nat(file, trackid)
+                            self.tagger.nats.update()
+                    if nats and not self.get_num_unmatched_files():
+                        self.tagger.remove_album(self)
+                        error = False
             else:
                 try:
                     parsed = self._parse_release(document)
@@ -175,17 +147,17 @@ class Album(DataObject, Item):
                 self._finalize_loading(error)
 
     def _parse_release_group(self, document):
-        releases = document.metadata[0].release_group[0].release_list[0].release
-        for release in releases:
-            version = {}
-            version["mbid"] = release.id
-            if "date" in release.children:
-                version["date"] = release.date[0].text
-            if "country" in release.children:
-                version["country"] = release.country[0].text
-            version["tracks"] = " + ".join([m.track_list[0].count for m in release.medium_list[0].medium])
-            version["format"] = media_formats_from_node(release.medium_list[0])
-            self.other_versions.append(version)
+        for node in document.metadata[0].release_list[0].release:
+            v = {}
+            v["mbid"] = node.id
+            v["date"] = node.date[0].text if "date" in node.children else ""
+            v["country"] = node.country[0].text if "country" in node.children else ""
+            labels, catnums = label_info_from_node(node.label_info_list[0])
+            v["labels"] = ", ".join(set(labels))
+            v["catnums"] = ", ".join(set(catnums))
+            v["tracks"] = " + ".join([m.track_list[0].count for m in node.medium_list[0].medium])
+            v["format"] = media_formats_from_node(node.medium_list[0])
+            self.other_versions.append(v)
         self.other_versions.sort(key=lambda x: x["date"])
 
     def _release_group_request_finished(self, document, http, error):
@@ -200,7 +172,7 @@ class Album(DataObject, Item):
                     self.log.error(traceback.format_exc())
         finally:
             self.rgloaded = True
-            self.emit(QtCore.SIGNAL("release_group_loaded"))
+            self.release_group_loaded.emit()
 
     def _finalize_loading(self, error):
         if error:
@@ -211,12 +183,54 @@ class Album(DataObject, Item):
             self.update()
             return
 
-        # Run metadata plugins
-        while self._metadata_plugins:
-            try:
-                self._metadata_plugins.pop()()
-            except:
-                self.log.error(traceback.format_exc())
+        if self._requests > 0:
+            return
+
+        if not self._tracks_loaded:
+            artists = set()
+            totalalbumtracks = 0
+
+            djmix_ars = {}
+            if hasattr(self._new_metadata, "_djmix_ars"):
+                djmix_ars = self._new_metadata._djmix_ars
+
+            for medium_node in self._release_node.medium_list[0].medium:
+                mm = Metadata()
+                mm.copy(self._new_metadata)
+                medium_to_metadata(medium_node, mm)
+                totalalbumtracks += int(mm["totaltracks"])
+
+                for dj in djmix_ars.get(mm["discnumber"], []):
+                    mm.add("djmixer", dj)
+
+                for track_node in medium_node.track_list[0].track:
+                    track = Track(track_node.recording[0].id, self)
+                    self._new_tracks.append(track)
+
+                    # Get track metadata
+                    tm = track.metadata
+                    tm.copy(mm)
+                    track_to_metadata(track_node, track, self.config)
+                    track._customize_metadata()
+
+                    self._new_metadata.length += tm.length
+                    artists.add(tm["musicbrainz_artistid"])
+
+                    # Run track metadata plugins
+                    try:
+                        run_track_metadata_processors(self, tm, self._release_node, track_node)
+                    except:
+                        self.log.error(traceback.format_exc())
+
+            totalalbumtracks = str(totalalbumtracks)
+
+            for track in self._new_tracks:
+                track.metadata["~totalalbumtracks"] = totalalbumtracks
+                if len(artists) > 1:
+                    track.metadata["compilation"] = "1"
+
+            del self._release_node
+            self._tracks_loaded = True
 
         if not self._requests:
             # Prepare parser for user's script
@@ -224,8 +238,8 @@ class Album(DataObject, Item):
                 script = self.config.setting["tagger_script"]
                 if script:
                     parser = ScriptParser()
-                    # Run tagger script for each track
                     for track in self._new_tracks:
+                        # Run tagger script for each track
                         try:
                             parser.eval(script, track.metadata)
                         except:
@@ -260,14 +274,18 @@ class Album(DataObject, Item):
             return
         self.tagger.window.set_statusbar_message('Loading album %s...', self.id)
         self.loaded = False
+        self.rgloaded = False
+        self.rgid = None
+        self.other_versions = []
         self.metadata.clear()
+        self.folksonomy_tags.clear()
         self.metadata['album'] = _("[loading album information]")
         self.update()
         self._new_metadata = Metadata()
         self._new_tracks = []
         self._requests = 1
         require_authentication = False
-        inc = ['release-groups', 'media', 'recordings', 'puids', 'artist-credits', 'labels', 'isrcs']
+        inc = ['release-groups', 'media', 'recordings', 'puids', 'artist-credits', 'artists', 'aliases', 'labels', 'isrcs']
         if self.config.setting['release_ars'] or self.config.setting['track_ars']:
             inc += ['artist-rels', 'release-rels', 'url-rels', 'recording-rels', 'work-rels']
             if self.config.setting['track_ars']:
@@ -281,8 +299,9 @@ class Album(DataObject, Item):
         if self.config.setting['enable_ratings']:
             require_authentication = True
             inc += ['user-ratings']
-        self.tagger.xmlws.get_release_by_id(self.id, self._release_request_finished, inc=inc,
-                mblogin=require_authentication)
+        self.load_task = self.tagger.xmlws.get_release_by_id(
+            self.id, self._release_request_finished, inc=inc,
+            mblogin=require_authentication)
 
     def run_when_loaded(self, func):
         if self.loaded:
@@ -290,20 +309,28 @@ class Album(DataObject, Item):
         else:
             self._after_load_callbacks.put(func)
 
+    def stop_loading(self):
+        if self.load_task:
+            self.tagger.xmlws.remove_task(self.load_task)
+            self.load_task = None
+
     def update(self, update_tracks=True):
-        self.tagger.emit(QtCore.SIGNAL("album_updated"), self, update_tracks)
+        if self.item:
+            self.item.update(update_tracks)
 
     def _add_file(self, track, file):
         self._files += 1
-        self.update(False)
+        self.update(update_tracks=False)
 
     def _remove_file(self, track, file):
         self._files -= 1
-        self.update(False)
+        self.update(update_tracks=False)
 
     def match_files(self, files, use_trackid=True):
         """Match files to tracks on this album, based on metadata similarity or trackid."""
         for file in list(files):
+            if file.state == File.REMOVED:
+                continue
             matches = []
             trackid = file.metadata['musicbrainz_trackid']
             if use_trackid and mbid_validate(trackid):
@@ -321,6 +348,8 @@ class Album(DataObject, Item):
 
     def match_file(self, file, trackid=None):
         """Match the file on a track on this album, based on trackid or metadata similarity."""
+        if file.state == File.REMOVED:
+            return
         if trackid is not None:
             matches = self._get_trackid_matches(file, trackid)
             if matches:
@@ -353,7 +382,7 @@ class Album(DataObject, Item):
         return True
 
     def can_edit_tags(self):
-        return False
+        return True
 
     def can_analyze(self):
         return False
@@ -364,6 +393,9 @@ class Album(DataObject, Item):
     def can_refresh(self):
         return True
 
+    def is_album_like(self):
+        return True
+
     def get_num_matched_tracks(self):
         num = 0
         for track in self.tracks:
@@ -419,15 +451,27 @@ class Album(DataObject, Item):
         else:
             return ''
 
-    def switch_release_version(self, version):
-        self.id = version["mbid"]
-        self.load()
+    def switch_release_version(self, mbid):
+        if mbid == self.id:
+            return
+        for file in list(self.iterfiles(True)):
+            file.move(self.unmatched_files)
+        album = self.tagger.albums.get(mbid)
+        if album:
+            album.match_files(self.unmatched_files.files)
+            album.update()
+            self.tagger.remove_album(self)
+        else:
+            del self.tagger.albums[self.id]
+            self.id = mbid
+            self.tagger.albums[mbid] = self
+            self.load()
 
 
 class NatAlbum(Album):
 
     def __init__(self):
-        super(NatAlbum, self).__init__(id="NATS")
+        Album.__init__(self, id="NATS")
         self.loaded = True
         self.update()
 
@@ -437,7 +481,10 @@ class NatAlbum(Album):
             track.metadata["album"] = self.metadata["album"]
             for file in track.linked_files:
                 track.update_file_metadata(file)
-        super(NatAlbum, self).update(update_tracks)
+        Album.update(self, update_tracks)
 
     def _finalize_loading(self, error):
         self.update()
+
+    def can_browser_lookup(self):
+        return False
