@@ -26,6 +26,7 @@ import re
 import unicodedata
 import traceback
 from PyQt4 import QtCore
+from picard.track import Track
 from picard.mbxml import artist_credit_from_node
 from picard.metadata import Metadata
 from picard.ui.item import Item
@@ -63,6 +64,17 @@ class File(LockableObject, Item):
     ERROR = 3
     REMOVED = 4
 
+    comparison_weights = {
+        "title": 13,
+        "artist": 4,
+        "album": 5,
+        "length": 10,
+        "totaltracks": 4,
+        "releasetype": 20,
+        "releasecountry": 2,
+        "format": 2,
+    }
+
     def __init__(self, filename):
         super(File, self).__init__()
         self.id = self.new_id()
@@ -74,6 +86,7 @@ class File(LockableObject, Item):
 
         self.orig_metadata = Metadata()
         self.user_metadata = Metadata()
+        self.saved_metadata = Metadata()
         self.metadata = self.user_metadata
 
         self.similarity = 1.0
@@ -81,10 +94,6 @@ class File(LockableObject, Item):
 
         self.lookup_task = None
 
-        self.comparison_weights = {"title": 13, "artist": 4, "album": 5,
-            "length": 10, "totaltracks": 4, "releasetype": 20,
-            "releasecountry": 2, "format": 2}
-
     def __repr__(self):
         return '<File #%d %r>' % (self.id, self.base_filename)
 
@@ -104,14 +113,15 @@ class File(LockableObject, Item):
         else:
             self.error = None
             self.state = self.NORMAL
-            self._copy_metadata(result)
+            self._copy_loaded_metadata(result)
         self.update()
         return self
 
-    def _copy_metadata(self, metadata):
+    def _copy_loaded_metadata(self, metadata):
         filename, extension = os.path.splitext(self.base_filename)
         self.metadata.copy(metadata)
         self.metadata['~extension'] = extension[1:].lower()
+        self.metadata['~length'] = format_time(self.metadata.length)
         if 'title' not in self.metadata:
             self.metadata['title'] = filename
         if 'tracknumber' not in self.metadata:
@@ -125,6 +135,16 @@ class File(LockableObject, Item):
                     self.metadata['tracknumber'] = str(tracknumber)
         self.orig_metadata.copy(self.metadata)
 
+    def copy_metadata(self, metadata):
+        exceptions = ['musicip_puid', 'acoustid_id']
+        if self.config.setting['preserved_tags']:
+            exceptions.extend(re.split(r'\s+', self.config.setting['preserved_tags'].lower().strip()))
+        for tag in exceptions:
+            self.saved_metadata[tag] = self.metadata[tag]
+        self.metadata.copy(metadata)
+        for tag in exceptions:
+            self.metadata[tag] = self.saved_metadata.pop(tag)
+
     def has_error(self):
         return self.state == File.ERROR
 
@@ -145,18 +165,34 @@ class File(LockableObject, Item):
         """Save the metadata."""
         new_filename = old_filename
         if not settings["dont_write_tags"]:
+            encoded_old_filename = encode_filename(old_filename)
+            info = os.stat(encoded_old_filename)
             self._save(old_filename, metadata, settings)
+            if settings["preserve_timestamps"]:
+                try:
+                    os.utime(encoded_old_filename, (info.st_atime, info.st_mtime))
+                except OSError:
+                    self.log.warning("Couldn't preserve timestamp for %r", old_filename)
         # Rename files
         if settings["rename_files"] or settings["move_files"]:
             new_filename = self._rename(old_filename, metadata, settings)
         # Move extra files (images, playlists, etc.)
         if settings["move_files"] and settings["move_additional_files"]:
-            self._move_additional_files(old_filename, new_filename,
-                                        settings)
+            self._move_additional_files(old_filename, new_filename, settings)
         # Delete empty directories
         if settings["delete_empty_dirs"]:
+            dirname = encode_filename(os.path.dirname(old_filename))
             try:
-                os.removedirs(encode_filename(os.path.dirname(old_filename)))
+                self._rmdir(dirname)
+                head, tail = os.path.split(dirname)
+                if not tail:
+                    head, tail = os.path.split(head)
+                while head and tail:
+                    try:
+                        self._rmdir(head)
+                    except:
+                        break
+                    head, tail = os.path.split(head)
             except EnvironmentError:
                 pass
         # Save cover art images
@@ -164,6 +200,14 @@ class File(LockableObject, Item):
             self._save_images(new_filename, metadata, settings)
         return new_filename
 
+    @staticmethod
+    def _rmdir(dir):
+        junk_files = (".DS_Store", "desktop.ini", "Desktop.ini", "Thumbs.db")
+        if not set(os.listdir(dir)) - set(junk_files):
+            shutil.rmtree(dir, False)
+        else:
+            raise OSError
+
     @call_next
     def _saving_finished(self, next, result=None, error=None):
         old_filename = new_filename = self.filename
@@ -178,11 +222,14 @@ class File(LockableObject, Item):
             for info in ('~#bitrate', '~#sample_rate', '~#channels',
                          '~#bits_per_sample', '~format', '~extension'):
                 temp_info[info] = self.orig_metadata[info]
-            self.orig_metadata.copy(self.metadata)
+            if self.config.setting["clear_existing_tags"]:
+                self.orig_metadata.copy(self.metadata)
+            else:
+                self.orig_metadata.update(self.metadata)
             self.orig_metadata.length = length
+            self.orig_metadata['~length'] = format_time(length)
             for k, v in temp_info.items():
                 self.orig_metadata[k] = v
-            self.metadata.changed = False
             self.error = None
             self.clear_pending()
         return self, old_filename, new_filename
@@ -193,7 +240,11 @@ class File(LockableObject, Item):
 
     def _script_to_filename(self, format, file_metadata, settings):
         metadata = Metadata()
-        metadata.copy(file_metadata)
+        if self.config.setting["clear_existing_tags"]:
+            metadata.copy(file_metadata)
+        else:
+            metadata.copy(self.orig_metadata)
+            metadata.update(file_metadata)
         # make sure every metadata can safely be used in a path name
         for name in metadata.keys():
             if isinstance(metadata[name], basestring):
@@ -207,6 +258,8 @@ class File(LockableObject, Item):
             if isinstance(filename, unicode):
                 filename = unaccent(filename)
             filename = replace_non_ascii(filename)
+        # remove null characters
+        filename = filename.replace("\x00", "")
         return filename
 
     def _make_filename(self, filename, metadata, settings):
@@ -217,14 +270,11 @@ class File(LockableObject, Item):
                 new_dirname = os.path.normpath(os.path.join(os.path.dirname(filename), new_dirname))
         else:
             new_dirname = os.path.dirname(filename)
-        old_dirname = new_dirname
         new_filename, ext = os.path.splitext(os.path.basename(filename))
 
         if settings["rename_files"]:
             # expand the naming format
             format = settings['file_naming_format']
-            if settings['use_va_format'] and metadata['compilation'] == '1':
-                format = settings['va_file_naming_format']
             if len(format) > 0:
                 new_filename = self._script_to_filename(format, metadata, settings)
                 if not settings['move_files']:
@@ -354,10 +404,16 @@ class File(LockableObject, Item):
         return self.similarity == 1.0 and self.state == File.NORMAL
 
     def update(self, signal=True):
-        for name, values in self.metadata.rawitems():
+        names = set(self.metadata._items.keys())
+        names.update(self.orig_metadata._items.keys())
+        clear_existing_tags = self.config.setting["clear_existing_tags"]
+        for name in names:
             if not name.startswith('~') and self.supports_tag(name):
-                if self.orig_metadata.getall(name) != values:
-                    #print name, values, self.orig_metadata.getall(name)
+                new_values = self.metadata.getall(name)
+                if not (new_values or clear_existing_tags):
+                    continue
+                orig_values = self.orig_metadata.getall(name)
+                if orig_values != new_values:
                     self.similarity = self.orig_metadata.compare(self.metadata)
                     if self.state in (File.CHANGED, File.NORMAL):
                         self.state = File.CHANGED
@@ -368,7 +424,10 @@ class File(LockableObject, Item):
                 self.state = File.NORMAL
         if signal:
             self.log.debug("Updating file %r", self)
-            self.parent.update_file(self)
+            if self.item:
+                self.item.update()
+            if isinstance(self.parent, Track):
+                self.parent.update()
 
     def can_save(self):
         """Return if this object can be saved."""
@@ -392,6 +451,9 @@ class File(LockableObject, Item):
     def can_refresh(self):
         return False
 
+    def can_view_info(self):
+        return True
+
     def _info(self, metadata, file):
         if hasattr(file.info, 'length'):
             metadata.length = int(file.info.length * 1000)
@@ -408,7 +470,6 @@ class File(LockableObject, Item):
     def get_state(self):
         return self._state
 
-
     # in order to significantly speed up performance, the number of pending
     #  files is cached
     num_pending_files = 0
@@ -428,12 +489,9 @@ class File(LockableObject, Item):
 
     def column(self, column):
         m = self.metadata
-        if column == '~length':
-            return format_time(m.length), self.similarity
-        elif column == "title" and not m["title"]:
-            return self.base_filename, self.similarity
-        else:
-            return m[column], self.similarity
+        if column == "title" and not m["title"]:
+            return self.base_filename
+        return m[column]
 
     def _compare_to_track(self, track):
         """
