@@ -75,6 +75,8 @@ class MP4File(File):
         "----:com.apple.iTunes:MusicBrainz TRM Id": "musicbrainz_trmid",
         "----:com.apple.iTunes:MusicBrainz Work Id": "musicbrainz_workid",
         "----:com.apple.iTunes:MusicBrainz Release Group Id": "musicbrainz_releasegroupid",
+        "----:com.apple.iTunes:Acoustid Fingerprint": "acoustid_fingerprint",
+        "----:com.apple.iTunes:Acoustid Id": "acoustid_id",
         "----:com.apple.iTunes:ASIN": "asin",
         "----:com.apple.iTunes:BARCODE": "barcode",
         "----:com.apple.iTunes:PRODUCER": "producer",
@@ -87,6 +89,7 @@ class MP4File(File):
         "----:com.apple.iTunes:ISRC": "isrc",
         "----:com.apple.iTunes:MEDIA": "media",
         "----:com.apple.iTunes:LABEL": "label",
+        "----:com.apple.iTunes:LICENSE": "license",
         "----:com.apple.iTunes:CATALOGNUMBER": "catalognumber",
         "----:com.apple.iTunes:SUBTITLE": "subtitle",
         "----:com.apple.iTunes:DISCSUBTITLE": "discsubtitle",
@@ -96,6 +99,9 @@ class MP4File(File):
     }
     __r_freeform_tags = dict([(v, k) for k, v in __freeform_tags.iteritems()])
 
+    __other_supported_tags = ("discnumber", "tracknumber",
+                              "totaldiscs", "totaltracks")
+
     def _load(self, filename):
         self.log.debug("Loading file %r", filename)
         file = MP4(encode_filename(filename))
@@ -191,5 +197,7 @@ class MP4File(File):
         file.save()
 
     def supports_tag(self, name):
-        return name in self.__r_text_tags or name in self.__r_bool_tags or name in self.__r_freeform_tags
-
+        return name in self.__r_text_tags or name in self.__r_bool_tags\
+            or name in self.__r_freeform_tags\
+            or name in self.__other_supported_tags\
+            or name.startswith('lyrics:')
