@@ -19,12 +19,15 @@
 
 import re
 import unicodedata
-from picard.util import format_time, translate_artist
+from picard.util import format_time, translate_from_sortname
 from picard.const import RELEASE_FORMATS
 
 
+AMAZON_ASIN_URL_REGEX = re.compile(r'^http://(?:www.)?(.*?)(?:\:[0-9]+)?/.*/([0-9B][0-9A-Z]{9})(?:[^0-9A-Z]|$)')
+
 _artist_rel_types = {
     "composer": "composer",
+    "writer": "writer",
     "conductor": "conductor",
     "chorus master": "conductor",
     "performing orchestra": "performer:orchestra",
@@ -69,9 +72,8 @@ def _relations_to_metadata(relation_lists, m, config):
     for relation_list in relation_lists:
         if relation_list.target_type == 'artist':
             for relation in relation_list.relation:
-                value = relation.artist[0].name[0].text
-                if config.setting['translate_artist_names']:
-                    value = translate_artist(value, relation.artist[0].sort_name[0].text)
+                artist = relation.artist[0]
+                value = _translate_artist_node(artist, config)[0] or artist.name[0].text
                 reltype = relation.type
                 attribs = []
                 if 'attribute_list' in relation.children:
@@ -82,6 +84,12 @@ def _relations_to_metadata(relation_lists, m, config):
                     name = 'performer:' + _parse_attributes(attribs)
                 elif reltype == 'performer':
                     name = 'performer:' + _parse_attributes(attribs)
+                elif reltype == 'mix-DJ' and len(attribs) > 0:
+                    if not hasattr(m, "_djmix_ars"):
+                        m._djmix_ars = {}
+                    for attr in attribs:
+                        m._djmix_ars.setdefault(attr.split()[1], []).append(value)
+                    continue
                 else:
                     try:
                         name = _artist_rel_types[reltype]
@@ -92,31 +100,56 @@ def _relations_to_metadata(relation_lists, m, config):
         elif relation_list.target_type == 'work':
             for relation in relation_list.relation:
                 if relation.type == 'performance':
-                    work = relation.work[0]
-                    if 'relation_list' in work.children:
-                        _relations_to_metadata(work.relation_list, m, config)
-        # TODO: Release, Track, URL relations
+                    work_to_metadata(relation.work[0], m, config)
+        elif relation_list.target_type == 'url':
+            for relation in relation_list.relation:
+                if relation.type == 'amazon asin':
+                    url = relation.target[0].text
+                    match = AMAZON_ASIN_URL_REGEX.match(url)
+                    if match is not None and 'asin' not in m:
+                        m['asin'] = match.group(2)
+                if relation.type == 'license':
+                    url = relation.target[0].text
+                    m.add('license', url)
 
 
-def _set_artist_item(m, release, albumname, name, value):
-    if release:
-        m[albumname] = value
-        if name not in m:
-            m[name] = value
-    else:
-        m[name] = value
+def _translate_artist_node(node, config=None):
+    transl, translsort = None, None
+    if config and config.setting['translate_artist_names']:
+        locale = config.setting["artist_locale"]
+        lang = locale.split("_")[0]
+        if "alias_list" in node.children:
+            found_primary = found_locale = False
+            for alias in node.alias_list[0].alias:
+                if alias.attribs.get("type") != "Search hint" and "locale" in alias.attribs:
+                    if alias.locale == locale:
+                        transl, translsort = alias.text, alias.attribs["sort_name"]
+                        if alias.attribs.get("primary") == "primary":
+                            return (transl, translsort)
+                        found_locale = True
+                    elif alias.locale == lang and not (found_locale or found_primary):
+                        transl, translsort = alias.text, alias.attribs["sort_name"]
+                        if alias.attribs.get("primary") == "primary":
+                            found_primary = True
+        if lang == "en" and not transl:
+            transl = translate_from_sortname(node.name[0].text, node.sort_name[0].text)
+    return (transl, translsort)
 
 
-def artist_credit_from_node(node, config):
+def artist_credit_from_node(node, config=None):
     artist = ""
     artistsort = ""
     for credit in node.name_credit:
         a = credit.artist[0]
-        artistsort += a.sort_name[0].text
-        if 'name' in credit.children and not config.setting["standardize_artists"]:
-            artist += credit.name[0].text
+        transl, translsort = _translate_artist_node(a, config)
+        if transl:
+            artist += transl
         else:
-            artist += a.name[0].text
+            if 'name' in credit.children and not (config and config.setting["standardize_artists"]):
+                artist += credit.name[0].text
+            else:
+                artist += a.name[0].text
+        artistsort += translsort if translsort else a.sort_name[0].text
         if 'joinphrase' in credit.attribs:
             artist += credit.joinphrase
             artistsort += credit.joinphrase
@@ -125,10 +158,15 @@ def artist_credit_from_node(node, config):
 
 def artist_credit_to_metadata(node, m, config, release=False):
     ids = [n.artist[0].id for n in node.name_credit]
-    _set_artist_item(m, release, 'musicbrainz_albumartistid', 'musicbrainz_artistid', ids)
     artist, artistsort = artist_credit_from_node(node, config)
-    _set_artist_item(m, release, 'albumartist', 'artist', artist)
-    _set_artist_item(m, release, 'albumartistsort', 'artistsort', artistsort)
+    if release:
+        m["musicbrainz_albumartistid"] = ids
+        m["albumartist"] = artist
+        m["albumartistsort"] = artistsort
+    else:
+        m["musicbrainz_artistid"] = ids
+        m["artist"] = artist
+        m["artistsort"] = artistsort
 
 
 def label_info_from_node(node):
@@ -144,37 +182,44 @@ def label_info_from_node(node):
 
 
 def media_formats_from_node(node):
-    formats = {}
+    formats_count = {}
+    formats_order = []
     for medium in node.medium:
         if "format" in medium.children:
             text = medium.format[0].text
-            formats.setdefault(text, 0)
-            formats[text] += 1
-    if formats:
-        return " + ".join([(str(j) + u"×" if j > 1 else "") + RELEASE_FORMATS[i]
-            for i, j in formats.items()])
-    else:
-        return ""
+        else:
+            text = "(unknown)"
+        if text in formats_count:
+            formats_count[text] += 1
+        else:
+            formats_count[text] = 1
+            formats_order.append(text)
+    formats = []
+    for format in formats_order:
+        count = formats_count[format]
+        format = RELEASE_FORMATS.get(format, format)
+        if count > 1:
+            format = str(count) + u"×" + format
+        formats.append(format)
+    return " + ".join(formats)
 
 
 def track_to_metadata(node, track, config):
     m = track.metadata
     recording_to_metadata(node.recording[0], track, config)
-    transl = m['releasestatus'] == "pseudo-release"
     # overwrite with data we have on the track
     for name, nodes in node.children.iteritems():
         if not nodes:
             continue
         if name == 'title':
-            if not config.setting["standardize_tracks"] or transl:
-                m['title'] = nodes[0].text
+            m['title'] = nodes[0].text
         elif name == 'position':
             m['tracknumber'] = nodes[0].text
         elif name == 'length' and nodes[0].text:
             m.length = int(nodes[0].text)
         elif name == 'artist_credit':
-            if not config.setting["standardize_artists"] or transl:
-                artist_credit_to_metadata(nodes[0], m, config)
+            artist_credit_to_metadata(nodes[0], m, config)
+    m['~length'] = format_time(m.length)
 
 
 def recording_to_metadata(node, track, config):
@@ -202,31 +247,45 @@ def recording_to_metadata(node, track, config):
             add_isrcs_to_metadata(nodes[0], m)
         elif name == 'user_rating':
             m['~rating'] = nodes[0].text
+    m['~length'] = format_time(m.length)
+
+def work_to_metadata(work, m, config):
+    m.add("musicbrainz_workid", work.attribs['id'])
+    if 'relation_list' in work.children:
+        _relations_to_metadata(work.relation_list, m, config)
+
+def medium_to_metadata(node, m):
+    for name, nodes in node.children.iteritems():
+        if not nodes:
+            continue
+        if name == 'position':
+            m['discnumber'] = nodes[0].text
+        elif name == 'track_list':
+            m['totaltracks'] = nodes[0].count
+        elif name == 'title':
+            m['discsubtitle'] = nodes[0].text
+        elif name == 'format':
+            m['media'] = nodes[0].text
 
 
 def release_to_metadata(node, m, config, album=None):
     """Make metadata dict from a XML 'release' node."""
     m['musicbrainz_albumid'] = node.attribs['id']
-
-    if "status" in node.children:
-        m['releasestatus'] = node.status[0].text.lower()
-    transl = m['releasestatus'] == "pseudo-release"
-
     for name, nodes in node.children.iteritems():
         if not nodes:
             continue
         if name == 'release_group':
             release_group_to_metadata(nodes[0], m, config, album)
+        elif name == 'status':
+            m['releasestatus'] = nodes[0].text.lower()
         elif name == 'title':
-            if not config.setting["standardize_releases"] or transl:
-                m['album'] = nodes[0].text
+            m['album'] = nodes[0].text
         elif name == 'disambiguation':
             m['~releasecomment'] = nodes[0].text
         elif name == 'asin':
             m['asin'] = nodes[0].text
         elif name == 'artist_credit':
-            if not config.setting["standardize_artists"] or transl:
-                artist_credit_to_metadata(nodes[0], m, config, release=True)
+            artist_credit_to_metadata(nodes[0], m, config, release=True)
         elif name == 'date':
             m['date'] = nodes[0].text
         elif name == 'country':
@@ -252,19 +311,11 @@ def release_group_to_metadata(node, m, config, album=None):
     """Make metadata dict from a XML 'release-group' node taken from inside a 'release' node."""
     if 'type' in node.attribs:
         m['releasetype'] = node.type.lower()
-    transl = m['releasestatus'] == "pseudo-release"
-
     for name, nodes in node.children.iteritems():
         if not nodes:
             continue
-        if name == 'title':
-            if config.setting["standardize_releases"] and not transl:
-                m['album'] = node.title[0].text
-        elif name == 'artist_credit':
-            if config.setting["standardize_artists"] and not transl:
-                artist_credit_to_metadata(nodes[0], m, config, release=True)
-        elif name == 'first_release_date':
-            m['~originaldate'] = nodes[0].text
+        if name == 'first_release_date':
+            m['originaldate'] = nodes[0].text
         elif name == 'tag_list':
             add_folksonomy_tags(nodes[0], album)
         elif name == 'user_tag_list':
