@@ -27,8 +27,7 @@ import os.path
 import shutil
 import signal
 import sys
-import traceback
-import time
+from collections import deque
 
 # Install gettext "noop" function.
 import __builtin__
@@ -52,59 +51,40 @@ shutil.copystat = _patched_shutil_copystat
 import picard.resources
 import picard.plugins
 
-from picard import musicdns, version_string, log
+from picard import musicdns, version_string, log, acoustid
 from picard.album import Album, NatAlbum
 from picard.browser.browser import BrowserIntegration
 from picard.browser.filelookup import FileLookup
 from picard.cluster import Cluster, ClusterList, UnmatchedFiles
 from picard.config import Config
-from picard.disc import Disc, DiscError
+from picard.disc import Disc
 from picard.file import File
 from picard.formats import open as open_file
-from picard.metadata import Metadata
 from picard.track import Track, NonAlbumTrack
-from picard.config import IntOption
-from picard.script import ScriptParser
 from picard.ui.mainwindow import MainWindow
 from picard.plugin import PluginManager
 from picard.puidmanager import PUIDManager
+from picard.acoustidmanager import AcoustIDManager
 from picard.util import (
     decode_filename,
     encode_filename,
-    make_short_filename,
-    replace_win32_incompat,
-    replace_non_ascii,
     sanitize_filename,
     icontheme,
-    webbrowser2,
-    pathcmp,
     partial,
     queue,
     thread,
-    mbid_validate
+    mbid_validate,
+    check_io_encoding
     )
 from picard.webservice import XmlWebService
 
 class Tagger(QtGui.QApplication):
 
-    """
-
-    Signals:
-      - file_state_changed
-      - file_updated(file)
-      - file_added_to_cluster(cluster, file)
-      - file_removed_from_cluster(cluster, file)
-
-      - cluster_added(cluster)
-      - cluster_removed(album, index)
-
-      - album_added(album)
-      - album_updated(album, update_tracks)
-      - album_removed(album, index)
-
-      - track_updated(track)
-
-    """
+    file_state_changed = QtCore.pyqtSignal(int)
+    cluster_added = QtCore.pyqtSignal(Cluster)
+    cluster_removed = QtCore.pyqtSignal(Cluster)
+    album_added = QtCore.pyqtSignal(Album)
+    album_removed = QtCore.pyqtSignal(Album)
 
     __instance = None
 
@@ -126,25 +106,17 @@ class Tagger(QtGui.QApplication):
         self.thread_pool = thread.ThreadPool(self)
 
         self.load_queue = queue.Queue()
-        self.load_queue.run_item = thread.generic_run_item
-
         self.save_queue = queue.Queue()
-        self.save_queue.run_item = thread.generic_run_item
-
         self.analyze_queue = queue.Queue()
-        self.analyze_queue.run_item = analyze_thread_run_item
-        self.analyze_queue.next = self._lookup_puid
-
         self.other_queue = queue.Queue()
-        self.other_queue.run_item = thread.generic_run_item
 
         threads = self.thread_pool.threads
-        threads.append(thread.Thread(self.thread_pool, [self.load_queue,
-                                                        self.other_queue]))
-        threads.append(thread.Thread(self.thread_pool, [self.save_queue]))
-        threads.append(thread.Thread(self.thread_pool, [self.other_queue,
-                                                        self.load_queue]))
-        threads.append(thread.Thread(self.thread_pool, [self.analyze_queue]))
+        threads.append(thread.Thread(self.thread_pool, self.load_queue))
+        threads.append(thread.Thread(self.thread_pool, self.load_queue))
+        threads.append(thread.Thread(self.thread_pool, self.save_queue))
+        threads.append(thread.Thread(self.thread_pool, self.other_queue))
+        threads.append(thread.Thread(self.thread_pool, self.other_queue))
+        threads.append(thread.Thread(self.thread_pool, self.analyze_queue))
 
         self.thread_pool.start()
         self.stopping = False
@@ -173,6 +145,8 @@ class Tagger(QtGui.QApplication):
         QtCore.QObject.config = self.config
         QtCore.QObject.log = self.log
 
+        check_io_encoding()
+
         self.setup_gettext(localedir)
 
         self.xmlws = XmlWebService()
@@ -180,7 +154,8 @@ class Tagger(QtGui.QApplication):
         # Initialize fingerprinting
         self._ofa = musicdns.OFA()
         self._ofa.init()
-        self.analyze_queue.ofa = self._ofa
+        self._acoustid = acoustid.AcoustIDClient()
+        self._acoustid.init()
 
         # Load plugins
         self.pluginmanager = PluginManager()
@@ -194,19 +169,41 @@ class Tagger(QtGui.QApplication):
             self.pluginmanager.load_plugindir(os.path.join(os.path.dirname(__file__), "plugins"))
 
         self.puidmanager = PUIDManager()
-
+        self.acoustidmanager = AcoustIDManager()
         self.browser_integration = BrowserIntegration()
 
         self.files = {}
-
         self.clusters = ClusterList()
-        self.albums = []
-        self.albumids = {}
-
+        self.albums = {}
+        self.mbid_redirects = {}
         self.unmatched_files = UnmatchedFiles()
+        self.nats = None
         self.window = MainWindow()
 
-        self.nats = None
+        def remove_va_file_naming_format(merge=True):
+            if merge:
+                self.config.setting["file_naming_format"] = \
+                    "$if($eq(%compilation%,1),\n$noop(Various Artist albums)\n"+\
+                    "%s,\n$noop(Single Artist Albums)\n%s)" %\
+                    (self.config.setting["va_file_naming_format"].toString(),
+                     self.config.setting["file_naming_format"])
+            self.config.setting.remove("va_file_naming_format")
+            self.config.setting.remove("use_va_format")
+
+        if "va_file_naming_format" in self.config.setting\
+                and "use_va_format" in self.config.setting:
+            if self.config.setting["use_va_format"].toBool():
+                remove_va_file_naming_format()
+                self.window.show_va_removal_notice()
+            elif self.config.setting["va_file_naming_format"].toString() !=\
+                r"$if2(%albumartist%,%artist%)/%album%/$if($gt(%totaldiscs%,1),%discnumber%-,)$num(%tracknumber%,2) %artist% - %title%":
+                    if self.window.confirm_va_removal():
+                        remove_va_file_naming_format(merge=False)
+                    else:
+                        remove_va_file_naming_format()
+            else:
+                # default format, disabled
+                remove_va_file_naming_format(merge=False)
 
     def setup_gettext(self, localedir):
         """Setup locales, load translations, install gettext functions."""
@@ -264,8 +261,8 @@ class Tagger(QtGui.QApplication):
     def create_nats(self):
         if self.nats is None:
             self.nats = NatAlbum()
-            self.albums.append(self.nats)
-            self.emit(QtCore.SIGNAL("album_added"), self.nats)
+            self.albums["NATS"] = self.nats
+            self.album_added.emit(self.nats)
         return self.nats
 
     def move_file_to_nat(self, file, trackid, node=None):
@@ -274,11 +271,12 @@ class Tagger(QtGui.QApplication):
         nat = self.load_nat(trackid, node=node)
         nat.run_when_loaded(partial(file.move, nat))
         if nat.loaded:
-            self.emit(QtCore.SIGNAL("album_updated"), self.nats)
+            self.nats.update()
 
     def exit(self):
         self.stopping = True
         self._ofa.done()
+        self._acoustid.done()
         self.thread_pool.stop()
         self.browser_integration.stop()
         self.xmlws.stop()
@@ -318,15 +316,18 @@ class Tagger(QtGui.QApplication):
         if file is not None and error is None and not file.has_error():
             puid = file.metadata['musicip_puid']
             trackid = file.metadata['musicbrainz_trackid']
-            albumid = file.metadata['musicbrainz_albumid']
             self.puidmanager.add(puid, trackid)
-            if mbid_validate(albumid):
-                if mbid_validate(trackid):
-                    self.move_file_to_track(file, albumid, trackid)
-                else:
-                    self.move_file_to_album(file, albumid)
-            elif mbid_validate(trackid):
-                self.move_file_to_nat(file, trackid)
+            if not self.config.setting["ignore_file_mbids"]:
+                albumid = file.metadata['musicbrainz_albumid']
+                if mbid_validate(albumid):
+                    if mbid_validate(trackid):
+                        self.move_file_to_track(file, albumid, trackid)
+                    else:
+                        self.move_file_to_album(file, albumid)
+                elif mbid_validate(trackid):
+                    self.move_file_to_nat(file, trackid)
+                elif self.config.setting['analyze_new_files']:
+                    self.analyze([file])
             elif self.config.setting['analyze_new_files']:
                 self.analyze([file])
 
@@ -347,17 +348,16 @@ class Tagger(QtGui.QApplication):
                 file.load(self._file_loaded)
 
     def process_directory_listing(self, root, queue, result=None, error=None):
-        delay = 10
         try:
             # Read directory listing
             if result is not None and error is None:
                 files = []
-                directories = []
+                directories = deque()
                 try:
                     for path in result:
                         path = os.path.join(root, path)
                         if os.path.isdir(path):
-                            directories.append(path)
+                            directories.appendleft(path)
                         else:
                             try:
                                 files.append(decode_filename(path))
@@ -367,25 +367,22 @@ class Tagger(QtGui.QApplication):
                 finally:
                     if files:
                         self.add_files(files)
-                    delay = min(25 * len(files), 500)
-                    queue = directories + queue
+                    queue.extendleft(directories)
         finally:
             # Scan next directory in the queue
             try:
-                path = queue.pop(0)
+                path = queue.popleft()
             except IndexError: pass
             else:
-                func = partial(self.other_queue.put,
-                               (partial(os.listdir, path),
-                                partial(self.process_directory_listing,
-                                        path, queue),
-                                QtCore.Qt.LowEventPriority))
-                QtCore.QTimer.singleShot(delay, func)
+                self.other_queue.put((
+                    partial(os.listdir, path),
+                    partial(self.process_directory_listing, path, queue),
+                    QtCore.Qt.LowEventPriority))
 
     def add_directory(self, path):
         path = encode_filename(path)
         self.other_queue.put((partial(os.listdir, path),
-                              partial(self.process_directory_listing, path, []),
+                              partial(self.process_directory_listing, path, deque()),
                               QtCore.Qt.LowEventPriority))
 
     def get_file_by_id(self, id):
@@ -410,29 +407,33 @@ class Tagger(QtGui.QApplication):
         lookup = self.get_file_lookup()
         getattr(lookup, type + "Search")(text, adv)
 
-    def lookup(self, metadata):
-        """Lookup the metadata on the MusicBrainz website."""
+    def browser_lookup(self, item):
+        """Lookup the object's metadata on the MusicBrainz website."""
         lookup = self.get_file_lookup()
+        metadata = item.metadata
         albumid = metadata["musicbrainz_albumid"]
         trackid = metadata["musicbrainz_trackid"]
-        if trackid:
+        # Only lookup via MB IDs if matched to a DataObject; otherwise ignore and use metadata details
+        if isinstance(item, Track) and trackid:
             lookup.trackLookup(trackid)
-        elif albumid:
+        elif isinstance(item, Album) and albumid:
             lookup.albumLookup(albumid)
         else:
-            lookup.tagLookup(metadata["artist"], metadata["album"],
-                             metadata["title"], metadata["tracknumber"],
-                             str(metadata.length),
-                             metadata["~filename"], metadata["musicip_puid"])
+            lookup.tagLookup(
+                metadata["albumartist"] if item.is_album_like() else metadata["artist"],
+                metadata["album"],
+                metadata["title"],
+                metadata["tracknumber"],
+                '' if item.is_album_like() else str(metadata.length),
+                item.filename if isinstance(item, File) else '',
+                metadata["musicip_puid"])
 
     def get_files_from_objects(self, objects, save=False):
         """Return list of files from list of albums, clusters, tracks or files."""
-        files = []
+        files = set()
         for obj in objects:
-            for file in obj.iterfiles(save):
-                if file not in files:
-                    files.append(file)
-        return files
+            files.update(obj.iterfiles(save))
+        return list(files)
 
     def _file_saved(self, result=None, error=None):
         if error is None:
@@ -447,14 +448,13 @@ class Tagger(QtGui.QApplication):
             file.save(self._file_saved, self.tagger.config.setting)
 
     def load_album(self, id, discid=None):
-        if id in self.albumids:
-            id = self.albumids[id]
-        album = self.get_album_by_id(id)
+        id = self.mbid_redirects.get(id, id)
+        album = self.albums.get(id)
         if album:
             return album
         album = Album(id, discid=discid)
-        self.albums.append(album)
-        self.emit(QtCore.SIGNAL("album_added"), album)
+        self.albums[id] = album
+        self.album_added.emit(album)
         album.load()
         return album
 
@@ -464,12 +464,6 @@ class Tagger(QtGui.QApplication):
         else:
             album.load()
 
-    def get_album_by_id(self, id):
-        for album in self.albums:
-            if album.id == id:
-                return album
-        return None
-
     def load_nat(self, id, node=None):
         self.create_nats()
         nat = self.get_nat_by_id(id)
@@ -495,16 +489,20 @@ class Tagger(QtGui.QApplication):
         for file in files:
             if self.files.has_key(file.filename):
                 file.clear_lookup_task()
-                self.analyze_queue.remove(file.filename)
+                self._ofa.stop_analyze(file)
+                self._acoustid.stop_analyze(file)
                 del self.files[file.filename]
                 file.remove(from_parent)
 
     def remove_album(self, album):
         """Remove the specified album."""
         self.log.debug("Removing %r", album)
+        album.stop_loading()
         self.remove_files(self.get_files_from_objects([album]))
-        self.albums.remove(album)
-        self.emit(QtCore.SIGNAL("album_removed"), album)
+        del self.albums[album.id]
+        if album == self.nats:
+            self.nats = None
+        self.album_removed.emit(album)
 
     def remove_cluster(self, cluster):
         """Remove the specified cluster."""
@@ -515,7 +513,7 @@ class Tagger(QtGui.QApplication):
             cluster.clear_lookup_task()
             self.remove_files(files, from_parent=False)
             self.clusters.remove(cluster)
-            self.emit(QtCore.SIGNAL("cluster_removed"), cluster)
+            self.cluster_removed.emit(cluster)
 
     def remove(self, objects):
         """Remove the specified objects."""
@@ -565,12 +563,19 @@ class Tagger(QtGui.QApplication):
                 self.window.set_statusbar_message(N_("Could not find PUID for file %s"), file.filename)
                 file.clear_pending()
 
+    @property
+    def use_acoustid(self):
+        return self.config.setting["fingerprinting_system"] == "acoustid"
+
     def analyze(self, objs):
         """Analyze the file(s)."""
         files = self.get_files_from_objects(objs)
         for file in files:
             file.set_pending()
-            self._ofa.analyze(file, partial(self._lookup_puid, file))
+            if self.use_acoustid:
+                self._acoustid.analyze(file, partial(file._lookup_finished, 'acoustid'))
+            else:
+                self._ofa.analyze(file, partial(self._lookup_puid, file))
 
     # =======================================================================
     #  Metadata-based lookups
@@ -609,7 +614,7 @@ class Tagger(QtGui.QApplication):
                 return cluster
         cluster = Cluster(name, artist)
         self.clusters.append(cluster)
-        self.emit(QtCore.SIGNAL("cluster_added"), cluster)
+        self.cluster_added.emit(cluster)
         return cluster
 
     # =======================================================================
@@ -642,18 +647,6 @@ class Tagger(QtGui.QApplication):
     def num_pending_files(self):
         return len([file for file in self.files.values() if file.state == File.PENDING])
 
-def analyze_thread_run_item(thread, queue, filename):
-    next = partial(queue.ofa._lookup_fingerprint, queue.next, filename)
-    priority = QtCore.Qt.LowEventPriority + 1
-    try:
-        result = queue.ofa.calculate_fingerprint(filename)
-    except:
-        import traceback
-        thread.log.error(traceback.format_exc())
-        thread.to_main(next, priority, error=sys.exc_info()[1])
-    else:
-        thread.to_main(next, priority, result=result)
-
 def help():
     print """Usage: %s [OPTIONS] [FILE] [FILE] ...
 
