@@ -19,13 +19,13 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 from PyQt4 import QtCore
-from picard.metadata import Metadata
+from picard.metadata import Metadata, run_track_metadata_processors
 from picard.dataobj import DataObject
-from picard.util import format_time, translate_artist, asciipunct, partial
+from picard.util import format_time, asciipunct, partial
 from picard.mbxml import recording_to_metadata
 from picard.script import ScriptParser
 from picard.const import VARIOUS_ARTISTS_ID
-import string
+from picard.ui.item import Item
 import traceback
 
 
@@ -36,7 +36,7 @@ _TRANSLATE_TAGS = {
 }
 
 
-class Track(DataObject):
+class Track(DataObject, Item):
 
     def __init__(self, id, album=None):
         DataObject.__init__(self, id)
@@ -58,10 +58,7 @@ class Track(DataObject):
     def update_file_metadata(self, file):
         if file not in self.linked_files:
             return
-        file.saved_metadata.copy(file.metadata)
-        file.metadata.copy(self.metadata)
-        if 'musicip_puid' in file.saved_metadata:
-            file.metadata['musicip_puid'] = file.saved_metadata['musicip_puid']
+        file.copy_metadata(self.metadata)
         file.metadata['~extension'] = file.orig_metadata['~extension']
         file.metadata.changed = True
         file.update(signal=False)
@@ -72,15 +69,13 @@ class Track(DataObject):
             return
         self.linked_files.remove(file)
         self.num_linked_files -= 1
-        file.metadata.copy(file.saved_metadata)
+        file.copy_metadata(file.orig_metadata)
         self.album._remove_file(self, file)
         self.update()
 
-    def update_file(self, file):
-        self.update()
-
     def update(self):
-        self.tagger.emit(QtCore.SIGNAL("track_updated"), self)
+        if self.item:
+            self.item.update()
 
     def iterfiles(self, save=False):
         for file in self.linked_files:
@@ -105,59 +100,33 @@ class Track(DataObject):
 
     def can_edit_tags(self):
         """Return if this object supports tag editing."""
-        for file in self.linked_files:
-            if file.can_edit_tags():
-                return True
-        return False
+        return True
 
-    def can_analyze(self):
-        """Return if this object can be fingerprinted."""
-        return False
-
-    def can_autotag(self):
-        return False
-
-    def can_refresh(self):
-        return False
-
-    def similarity(self):
-        if self.num_linked_files == 1:
-            return self.linked_files[0].similarity
-        else:
-            return 1
+    def can_view_info(self):
+        return self.num_linked_files == 1
 
     def column(self, column):
         m = self.metadata
-        similarity = self.similarity()
         if column == 'title':
-            prefix = "%s-" % m['discnumber'] if m['totaldiscs'] != "1" else ""
-            return u"%s%s  %s" % (prefix, m['tracknumber'].zfill(2), m['title']), similarity
-        elif column == '~length':
-            return format_time(m.length), similarity
-        else:
-            return m[column], similarity
+            prefix = "%s-" % m['discnumber'] if m['discnumber'] and m['totaldiscs'] != "1" else ""
+            return u"%s%s  %s" % (prefix, m['tracknumber'].zfill(2), m['title'])
+        return m[column]
 
-    def _customize_metadata(self, ignore_tags=None):
+    def _customize_metadata(self):
         tm = self.metadata
 
-        # 'Translate' artist name
-        if self.config.setting['translate_artist_names']:
-            tm['artist'] = translate_artist(tm['artist'], tm['artistsort'])
-
         # Custom VA name
         if tm['musicbrainz_artistid'] == VARIOUS_ARTISTS_ID:
             tm['artistsort'] = tm['artist'] = self.config.setting['va_name']
 
         if self.config.setting['folksonomy_tags']:
-            if ignore_tags is None:
-                ignore_tags = [s.strip() for s in self.config.setting['ignore_tags'].split(',')]
-            self._convert_folksonomy_tags_to_genre(ignore_tags)
+            self._convert_folksonomy_tags_to_genre()
 
         # Convert Unicode punctuation
         if self.config.setting['convert_punctuation']:
             tm.apply_func(asciipunct)
 
-    def _convert_folksonomy_tags_to_genre(self, ignore_tags):
+    def _convert_folksonomy_tags_to_genre(self):
         # Combine release and track tags
         tags = dict(self.folksonomy_tags)
         for name, count in self.album.folksonomy_tags.iteritems():
@@ -174,6 +143,7 @@ class Track(DataObject):
         # And generate the genre metadata tag
         maxtags = self.config.setting['max_tags']
         minusage = self.config.setting['min_tag_usage']
+        ignore_tags = self.config.setting['ignore_tags']
         genre = []
         for usage, name in taglist[:maxtags]:
             if name in ignore_tags:
@@ -187,6 +157,7 @@ class Track(DataObject):
             genre = [join_tags.join(genre)]
         self.metadata['genre'] = genre
 
+
 class NonAlbumTrack(Track):
 
     def __init__(self, id):
@@ -201,13 +172,14 @@ class NonAlbumTrack(Track):
 
     def column(self, column):
         if column == "title":
-            return u"%s" % self.metadata["title"], self.similarity()
-        else:
-            return super(NonAlbumTrack, self).column(column)
+            return self.metadata["title"]
+        return Track.column(self, column)
 
     def load(self):
-        inc = ["artist-credits"]
+        inc = ["artist-credits", "artists", "aliases"]
         mblogin = False
+        if self.config.setting["track_ars"]:
+            inc += ["artist-rels", "url-rels", "recording-rels", "work-rels", "work-level-rels"]
         if self.config.setting["folksonomy_tags"]:
             if self.config.setting["only_my_tags"]:
                 mblogin = True
@@ -233,12 +205,18 @@ class NonAlbumTrack(Track):
 
     def _parse_recording(self, recording):
         recording_to_metadata(recording, self, self.config)
+        self._customize_metadata()
+        m = self.metadata
+        run_track_metadata_processors(self.album, m, None, recording)
         if self.config.setting["enable_tagger_script"]:
             script = self.config.setting["tagger_script"]
-            parser = ScriptParser()
-        else:
-            script = parser = None
-        self._customize_metadata(recording, None, script, parser)
+            if script:
+                parser = ScriptParser()
+                try:
+                    parser.eval(script, m)
+                except:
+                    self.log.error(traceback.format_exc())
+                m.strip_whitespace()
         self.loaded = True
         if self.callback:
             self.callback()
