@@ -28,14 +28,16 @@ from picard.util import encode_filename, icontheme, partial, webbrowser2
 from picard.config import Option, TextOption
 from picard.plugin import ExtensionPoint
 from picard.const import RELEASE_COUNTRIES
+from picard.ui.ratingwidget import RatingWidget
 
 
 class BaseAction(QtGui.QAction):
     NAME = "Unknown"
+    MENU = []
 
     def __init__(self):
         QtGui.QAction.__init__(self, self.NAME, None)
-        self.connect(self, QtCore.SIGNAL("triggered()"), self.__callback)
+        self.triggered.connect(self.__callback)
 
     def __callback(self):
         objs = self.tagger.window.panel.selected_objects()
@@ -47,6 +49,7 @@ class BaseAction(QtGui.QAction):
 
 _album_actions = ExtensionPoint()
 _cluster_actions = ExtensionPoint()
+_clusterlist_actions = ExtensionPoint()
 _track_actions = ExtensionPoint()
 _file_actions = ExtensionPoint()
 
@@ -56,13 +59,15 @@ def register_album_action(action):
 def register_cluster_action(action):
     _cluster_actions.register(action.__module__, action)
 
+def register_clusterlist_action(action):
+    _clusterlist_actions.register(action.__module__, action)
+
 def register_track_action(action):
     _track_actions.register(action.__module__, action)
 
 def register_file_action(action):
     _file_actions.register(action.__module__, action)
 
-
 def get_match_color(similarity, basecolor):
     c1 = (basecolor.red(), basecolor.green(), basecolor.blue())
     c2 = (223, 125, 125)
@@ -88,24 +93,31 @@ class MainPanel(QtGui.QSplitter):
         QtGui.QSplitter.__init__(self, parent)
         self.window = window
         self.create_icons()
-        self._object_to_item = {}
-        self._item_to_object = {}
         self.views = [FileTreeView(window, self), AlbumTreeView(window, self)]
+        self.views[0].itemSelectionChanged.connect(self.update_selection_0)
+        self.views[1].itemSelectionChanged.connect(self.update_selection_1)
         self._selected_view = 0
         self._ignore_selection_changes = False
-        self.connect(self.views[0], QtCore.SIGNAL("itemSelectionChanged()"), self.update_selection_0)
-        self.connect(self.views[1], QtCore.SIGNAL("itemSelectionChanged()"), self.update_selection_1)
-        self.file_colors = {
-            File.NORMAL: self.palette().text().color(),
+        self._selected_objects = set()
+
+        TreeItem.window = window
+        TreeItem.base_color = self.palette().base().color()
+        TreeItem.text_color = self.palette().text().color()
+        TrackItem.track_colors = {
+            File.NORMAL: self.config.setting["color_saved"],
+            File.CHANGED: TreeItem.text_color,
+            File.PENDING: self.config.setting["color_pending"],
+            File.ERROR: self.config.setting["color_error"],
+        }
+        FileItem.file_colors = {
+            File.NORMAL: TreeItem.text_color,
             File.CHANGED: self.config.setting["color_modified"],
             File.PENDING: self.config.setting["color_pending"],
             File.ERROR: self.config.setting["color_error"],
         }
-        self.connect(self.tagger, QtCore.SIGNAL("file_updated"), self.update_file)
-        self.connect(self.tagger, QtCore.SIGNAL("cluster_updated"), self.update_cluster)
-        self.connect(self.tagger, QtCore.SIGNAL("file_added_to_cluster"), self.add_file_to_cluster)
-        self.connect(self.tagger, QtCore.SIGNAL("files_added_to_cluster"), self.add_files_to_cluster)
-        self.connect(self.tagger, QtCore.SIGNAL("file_removed_from_cluster"), self.remove_file_from_cluster)
+
+    def selected_objects(self):
+        return list(self._selected_objects)
 
     def save_state(self):
         self.config.persist["splitter_state"] = self.saveState()
@@ -117,18 +129,17 @@ class MainPanel(QtGui.QSplitter):
 
     def create_icons(self):
         if hasattr(QtGui.QStyle, 'SP_DirIcon'):
-            self.icon_dir = self.style().standardIcon(QtGui.QStyle.SP_DirIcon)
+            ClusterItem.icon_dir = self.style().standardIcon(QtGui.QStyle.SP_DirIcon)
         else:
-            self.icon_dir = icontheme.lookup('folder', icontheme.ICON_SIZE_MENU)
-        self.icon_file = QtGui.QIcon(":/images/file.png")
-        self.icon_file_pending = QtGui.QIcon(":/images/file-pending.png")
-        self.icon_cd = icontheme.lookup('media-optical', icontheme.ICON_SIZE_MENU)
-        self.icon_cd_saved = icontheme.lookup('media-optical-saved', icontheme.ICON_SIZE_MENU)
-        self.icon_note = QtGui.QIcon(":/images/note.png")
-        self.icon_error = icontheme.lookup('dialog-error', icontheme.ICON_SIZE_MENU)
-        self.icon_saved = QtGui.QIcon(":/images/track-saved.png")
-        self.icon_plugins = icontheme.lookup('applications-system', icontheme.ICON_SIZE_MENU)
-        self.match_icons = [
+            ClusterItem.icon_dir = icontheme.lookup('folder', icontheme.ICON_SIZE_MENU)
+        AlbumItem.icon_cd = icontheme.lookup('media-optical', icontheme.ICON_SIZE_MENU)
+        AlbumItem.icon_cd_saved = icontheme.lookup('media-optical-saved', icontheme.ICON_SIZE_MENU)
+        TrackItem.icon_note = QtGui.QIcon(":/images/note.png")
+        FileItem.icon_file = QtGui.QIcon(":/images/file.png")
+        FileItem.icon_file_pending = QtGui.QIcon(":/images/file-pending.png")
+        FileItem.icon_error = icontheme.lookup('dialog-error', icontheme.ICON_SIZE_MENU)
+        FileItem.icon_saved = QtGui.QIcon(":/images/track-saved.png")
+        FileItem.match_icons = [
             QtGui.QIcon(":/images/match-50.png"),
             QtGui.QIcon(":/images/match-60.png"),
             QtGui.QIcon(":/images/match-70.png"),
@@ -136,7 +147,7 @@ class MainPanel(QtGui.QSplitter):
             QtGui.QIcon(":/images/match-90.png"),
             QtGui.QIcon(":/images/match-100.png"),
         ]
-        self.match_pending_icons = [
+        FileItem.match_pending_icons = [
             QtGui.QIcon(":/images/match-pending-50.png"),
             QtGui.QIcon(":/images/match-pending-60.png"),
             QtGui.QIcon(":/images/match-pending-70.png"),
@@ -144,15 +155,14 @@ class MainPanel(QtGui.QSplitter):
             QtGui.QIcon(":/images/match-pending-90.png"),
             QtGui.QIcon(":/images/match-pending-100.png"),
         ]
-
-    def selected_objects(self):
-        items = self.views[self._selected_view].selectedItems()
-        return map(self.object_from_item, items)
+        self.icon_plugins = icontheme.lookup('applications-system', icontheme.ICON_SIZE_MENU)
 
     def update_selection(self, i, j):
         self._selected_view = i
         self.views[j].clearSelection()
-        self.window.updateSelection(self.selected_objects())
+        self._selected_objects.clear()
+        self._selected_objects.update(item.obj for item in self.views[i].selectedItems())
+        self.window.update_selection(self.selected_objects())
 
     def update_selection_0(self):
         if not self._ignore_selection_changes:
@@ -166,131 +176,8 @@ class MainPanel(QtGui.QSplitter):
             self.update_selection(1, 0)
             self._ignore_selection_changes = False
 
-    def register_object(self, obj, item):
-        self._object_to_item[obj] = item
-        self._item_to_object[item] = obj
-
-    def update_object(self, obj, item):
-        oldobj = self._item_to_object[item]
-        if oldobj != obj:
-            self._object_to_item[obj] = item
-            self._item_to_object[item] = obj
-            if oldobj in self._object_to_item:
-                del self._object_to_item[oldobj]
-
-    def unregister_object(self, obj=None, item=None):
-        if obj is None and item is not None:
-            obj = self.object_from_item(item)
-        if obj is not None and item is None:
-            item = self.item_from_object(obj)
-        del self._object_to_item[obj]
-        del self._item_to_object[item]
-
-    def object_from_item(self, item):
-        return self._item_to_object[item]
-
-    def item_from_object(self, obj):
-        return self._object_to_item[obj]
-
-    def update_file(self, file, item=None):
-        if item is None:
-            try:
-                item = self.item_from_object(file)
-            except KeyError:
-                self.log.debug("Item for %r not found", file)
-                return
-        item.setIcon(0, self.decide_file_icon(file))
-        color = self.file_colors[file.state]
-        for i, column in enumerate(self.columns):
-            text, similarity = file.column(column[1])
-            item.setText(i, text)
-            item.setTextColor(i, color)
-            item.setBackgroundColor(i, get_match_color(similarity, self.palette().base().color()))
-        item.setData(1, QtCore.Qt.UserRole, QtCore.QVariant(file.metadata.length or 0))
-
-    def decide_file_icon(self, file):
-        if file.state == File.ERROR:
-            return self.icon_error
-        elif isinstance(file.parent, Track):
-            if file.state == File.NORMAL:
-                return self.icon_saved
-            elif file.state == File.PENDING:
-                return self.match_pending_icons[int(file.similarity * 5 + 0.5)]
-            else:
-                return self.match_icons[int(file.similarity * 5 + 0.5)]
-        elif file.state == File.PENDING:
-            return self.icon_file_pending
-        else:
-            return self.icon_file
-
-    def update_cluster(self, cluster, item=None):
-        if item is None:
-            try:
-                item = self.item_from_object(cluster)
-            except KeyError:
-                self.log.debug("Item for %r not found", cluster)
-                return
-        for i, column in enumerate(self.columns):
-            item.setText(i, cluster.column(column[1]))
-        item.setData(1, QtCore.Qt.UserRole, QtCore.QVariant(cluster.metadata.length or 0))
-        album = cluster.related_album
-        if cluster.special and album and album.loaded:
-            self.views[1].update_album(album, update_tracks=False)
-
-    def add_file_to_cluster(self, cluster, file):
-        try:
-            cluster_item = self.item_from_object(cluster)
-        except KeyError:
-            self.log.debug("Item for %r not found", cluster)
-            return
-        if cluster.hide_if_empty and cluster.files:
-            cluster_item.setHidden(False)
-        self.update_cluster(cluster, cluster_item)
-        item = SortTreeWidgetItem(cluster_item)
-        self.register_object(file, item)
-        self.update_file(file, item)
-
-    def add_files_to_cluster(self, cluster, files):
-        cluster_item = self.item_from_object(cluster)
-        if cluster.hide_if_empty and cluster.files:
-            cluster_item.setHidden(False)
-        self.update_cluster(cluster, cluster_item)
-        items = []
-        for file in files:
-            item = SortTreeWidgetItem()
-            self.register_object(file, item)
-            self.update_file(file, item)
-            items.append(item)
-        cluster_item.addChildren(items)
-
-    def remove_file_from_cluster(self, cluster, file):
-        try:
-            cluster_item = self.item_from_object(cluster)
-        except KeyError:
-            self.log.debug("Item for %r not found", cluster)
-            return
-        index = cluster_item.indexOfChild(self.item_from_object(file))
-        if cluster_item.takeChild(index):
-            self.unregister_object(file)
-        self.update_cluster(cluster, cluster_item)
-        if cluster.hide_if_empty and not cluster.files:
-            cluster_item.setHidden(True)
-
-
-class NoSortTreeWidgetItem(QtGui.QTreeWidgetItem):
-
-    def __lt__ (self, other):
-        return False
-
-
-class SortTreeWidgetItem(QtGui.QTreeWidgetItem):
-
-    def __lt__(self, other):
-        column = self.treeWidget().sortColumn()
-        if column == 1:
-            return self.data(1, QtCore.Qt.UserRole).toInt() < other.data(1, QtCore.Qt.UserRole).toInt()
-        else:
-            return self.text(column).toLower() < other.text(column).toLower()
+    def update_current_view(self):
+        self.update_selection(self._selected_view, abs(self._selected_view - 1))
 
 
 class BaseTreeView(QtGui.QTreeWidget):
@@ -308,10 +195,9 @@ class BaseTreeView(QtGui.QTreeWidget):
         QtGui.QTreeWidget.__init__(self, parent)
         self.window = window
         self.panel = parent
-        self.columns = self.panel.columns
 
-        self.numHeaderSections = len(self.columns)
-        self.setHeaderLabels([_(h) for h, n in self.columns])
+        self.numHeaderSections = len(MainPanel.columns)
+        self.setHeaderLabels([_(h) for h, n in MainPanel.columns])
         self.restore_state()
 
         self.setAcceptDrops(True)
@@ -326,50 +212,64 @@ class BaseTreeView(QtGui.QTreeWidget):
         self.setSortingEnabled(True)
 
         self.expand_all_action = QtGui.QAction(_("&Expand all"), self)
-        self.connect(self.expand_all_action, QtCore.SIGNAL("triggered()"), self.expandAll)
+        self.expand_all_action.triggered.connect(self.expandAll)
         self.collapse_all_action = QtGui.QAction(_("&Collapse all"), self)
-        self.connect(self.collapse_all_action, QtCore.SIGNAL("triggered()"), self.collapseAll)
-
-        self.connect(self, QtCore.SIGNAL("doubleClicked(QModelIndex)"), self.activate_item)
-
-    def _switch_release_version(self, album):
-        index = self.sender().data().toInt()[0]
-        album.switch_release_version(album.other_versions[index])
+        self.collapse_all_action.triggered.connect(self.collapseAll)
+        self.doubleClicked.connect(self.activate_item)
 
     def contextMenuEvent(self, event):
         item = self.itemAt(event.pos())
         if not item:
             return
-        obj = self.panel.object_from_item(item)
-
+        obj = item.obj
         plugin_actions = None
+        can_view_info = self.window.view_info_action.isEnabled()
         menu = QtGui.QMenu(self)
+
         if isinstance(obj, Track):
-            menu.addAction(self.window.edit_tags_action)
+            if can_view_info:
+                menu.addAction(self.window.view_info_action)
             plugin_actions = list(_track_actions)
             if obj.num_linked_files == 1:
+                menu.addAction(self.window.open_file_action)
+                menu.addAction(self.window.open_folder_action)
                 plugin_actions.extend(_file_actions)
+            menu.addAction(self.window.browser_lookup_action)
+            menu.addSeparator()
             if isinstance(obj, NonAlbumTrack):
                 menu.addAction(self.window.refresh_action)
         elif isinstance(obj, Cluster):
+            menu.addAction(self.window.browser_lookup_action)
+            menu.addSeparator()
             menu.addAction(self.window.autotag_action)
             menu.addAction(self.window.analyze_action)
             if isinstance(obj, UnmatchedFiles):
                 menu.addAction(self.window.cluster_action)
             plugin_actions = list(_cluster_actions)
+        elif isinstance(obj, ClusterList):
+            menu.addAction(self.window.autotag_action)
+            menu.addAction(self.window.analyze_action)
+            plugin_actions = list(_clusterlist_actions)
         elif isinstance(obj, File):
-            menu.addAction(self.window.edit_tags_action)
+            if can_view_info:
+                menu.addAction(self.window.view_info_action)
+            menu.addAction(self.window.open_file_action)
+            menu.addAction(self.window.open_folder_action)
+            menu.addAction(self.window.browser_lookup_action)
+            menu.addSeparator()
             menu.addAction(self.window.autotag_action)
             menu.addAction(self.window.analyze_action)
             plugin_actions = list(_file_actions)
         elif isinstance(obj, Album):
+            menu.addAction(self.window.browser_lookup_action)
+            menu.addSeparator()
             menu.addAction(self.window.refresh_action)
             plugin_actions = list(_album_actions)
 
         menu.addAction(self.window.save_action)
         menu.addAction(self.window.remove_action)
 
-        if isinstance(obj, Album) and not isinstance(obj, NatAlbum):
+        if isinstance(obj, Album) and not isinstance(obj, NatAlbum) and obj.loaded:
             releases_menu = QtGui.QMenu(_("&Other versions"), menu)
             menu.addSeparator()
             menu.addMenu(releases_menu)
@@ -378,43 +278,51 @@ class BaseTreeView(QtGui.QTreeWidget):
 
             def _add_other_versions():
                 releases_menu.removeAction(loading)
-                switch_release_version = partial(self._switch_release_version, obj)
                 actions = []
                 for i, version in enumerate(obj.other_versions):
-                    name = []
-                    if "date" in version and version["date"]:
-                        name.append(version["date"])
-                    if "country" in version:
-                        name.append(RELEASE_COUNTRIES.get(version["country"], version["country"]))
-                    name.append(version["tracks"])
-                    if "format" in version and version["format"]:
-                        name.append(version["format"])
-                    version_name = " / ".join(name).replace('&', '&&')
-                    action = releases_menu.addAction(version_name or _('[no release info]'))
-                    action.setData(QtCore.QVariant(i))
+                    keys = ("date", "country", "labels", "catnums", "tracks", "format")
+                    name = " / ".join([version[k] for k in keys if version[k]]).replace("&", "&&")
+                    if name == version["tracks"]:
+                        name = "%s / %s" % (_('[no release info]'), name)
+                    action = releases_menu.addAction(name)
                     action.setCheckable(True)
                     if obj.id == version["mbid"]:
                         action.setChecked(True)
-                    self.connect(action, QtCore.SIGNAL("triggered(bool)"), switch_release_version)
-                if releases_menu.isEmpty():
-                    action = releases_menu.addAction(_('No other versions'))
-                    action.setEnabled(False)
+                    action.triggered.connect(partial(obj.switch_release_version, version["mbid"]))
 
-            if not obj.rgloaded:
-                if obj.rgid:
-                    self.connect(obj, QtCore.SIGNAL("release_group_loaded"), _add_other_versions)
-                    self.tagger.xmlws.get_release_group_by_id(obj.rgid, obj._release_group_request_finished)
-            else:
+            if obj.rgloaded:
                 _add_other_versions()
+            elif obj.rgid:
+                obj.release_group_loaded.connect(_add_other_versions)
+                kwargs = {"release-group": obj.rgid, "limit": 100}
+                self.tagger.xmlws.browse_releases(obj._release_group_request_finished, **kwargs)
+
+        if self.config.setting["enable_ratings"] and \
+           len(self.window.selected_objects) == 1 and isinstance(obj, Track):
+            menu.addSeparator()
+            action = QtGui.QWidgetAction(menu)
+            action.setDefaultWidget(RatingWidget(menu, obj))
+            menu.addAction(action)
+            menu.addSeparator()
 
         if plugin_actions:
             plugin_menu = QtGui.QMenu(_("&Plugins"), menu)
-            plugin_menu.addActions(plugin_actions)
             plugin_menu.setIcon(self.panel.icon_plugins)
             menu.addSeparator()
             menu.addMenu(plugin_menu)
 
-        if isinstance(obj, Cluster) or isinstance(obj, Album):
+            plugin_menus = {}
+            for action in plugin_actions:
+                action_menu = plugin_menu
+                for index in xrange(1, len(action.MENU)):
+                    key = tuple(action.MENU[:index])
+                    try:
+                        action_menu = plugin_menus[key]
+                    except KeyError:
+                        action_menu = plugin_menus[key] = action_menu.addMenu(key[-1])
+                action_menu.addAction(action)
+
+        if isinstance(obj, Cluster) or isinstance(obj, ClusterList) or isinstance(obj, Album):
             menu.addAction(self.expand_all_action)
             menu.addAction(self.collapse_all_action)
 
@@ -474,7 +382,7 @@ class BaseTreeView(QtGui.QTreeWidget):
         album_ids = []
         file_ids = []
         for item in items:
-            obj = self.panel.object_from_item(item)
+            obj = item.obj
             if isinstance(obj, Album):
                 album_ids.append(str(obj.id))
             elif isinstance(obj, Track):
@@ -499,9 +407,8 @@ class BaseTreeView(QtGui.QTreeWidget):
             for file in files:
                 file.move(target)
         elif isinstance(target, File):
-            if target.parent:
-                for file in files:
-                    file.move(target.parent)
+            for file in files:
+                file.move(target.parent)
         elif isinstance(target, Album):
             self.tagger.move_files_to_album(files, album=target)
         elif isinstance(target, ClusterList):
@@ -553,7 +460,7 @@ class BaseTreeView(QtGui.QTreeWidget):
             else:
                 item = parent.child(index)
             if item is not None:
-                target = self.panel.object_from_item(item)
+                target = item.obj
         self.log.debug("Drop target = %r", target)
         handled = False
         # text/uri-list
@@ -572,188 +479,244 @@ class BaseTreeView(QtGui.QTreeWidget):
         # application/picard.album-list
         albums = data.data("application/picard.album-list")
         if albums:
+            if isinstance(self, FileTreeView) and target is None:
+                target = self.tagger.unmatched_files
             albums = [self.tagger.load_album(id) for id in str(albums).split("\n")]
             self.drop_albums(albums, target)
             handled = True
         return handled
 
     def activate_item(self, index):
-        obj = self.panel.object_from_item(self.itemFromIndex(index))
-        if obj.can_edit_tags():
-            self.window.edit_tags([obj])
+        obj = self.itemFromIndex(index).obj
+        if obj.can_view_info():
+            self.window.view_info()
 
     def add_cluster(self, cluster, parent_item=None):
         if parent_item is None:
             parent_item = self.clusters
-        if cluster.special:
-            cluster_item = NoSortTreeWidgetItem(parent_item)
-        else:
-            cluster_item = SortTreeWidgetItem(parent_item)
-        cluster_item.setIcon(0, self.panel.icon_dir)
-        self.panel.update_cluster(cluster, cluster_item)
-        self.panel.register_object(cluster, cluster_item)
-        for file in cluster.files:
-            item = SortTreeWidgetItem(cluster_item)
-            self.panel.register_object(file, item)
-            self.panel.update_file(file, item)
+        cluster_item = ClusterItem(cluster, not cluster.special, parent_item)
         if cluster.hide_if_empty and not cluster.files:
+            cluster_item.update()
             cluster_item.setHidden(True)
+        else:
+            cluster_item.add_files(cluster.files)
+
+    def currentChanged(self, current, previous):
+        QtGui.QTreeWidget.currentChanged(self, current, previous)
+        item = self.itemFromIndex(current)
+        if item and not item.isSelected():
+            self.setCurrentItem(item)
+
+    def moveCursor(self, action, modifiers):
+        if action in (QtGui.QAbstractItemView.MoveUp, QtGui.QAbstractItemView.MoveDown):
+            item = self.currentItem()
+            if item and not item.isSelected():
+                self.setCurrentItem(item)
+        return QtGui.QTreeWidget.moveCursor(self, action, modifiers)
+
 
 class FileTreeView(BaseTreeView):
 
     def __init__(self, window, parent=None):
         BaseTreeView.__init__(self, window, parent)
-        self.unmatched_files = NoSortTreeWidgetItem(self)
-        self.unmatched_files.setIcon(0, self.panel.icon_dir)
-        self.panel.register_object(self.tagger.unmatched_files, self.unmatched_files)
-        self.panel.update_cluster(self.tagger.unmatched_files)
+        self.unmatched_files = ClusterItem(self.tagger.unmatched_files, False, self)
+        self.unmatched_files.update()
         self.setItemExpanded(self.unmatched_files, True)
-        self.clusters = NoSortTreeWidgetItem(self)
+        self.clusters = ClusterItem(self.tagger.clusters, False, self)
         self.clusters.setText(0, _(u"Clusters"))
-        self.clusters.setIcon(0, self.panel.icon_dir)
-        self.panel.register_object(self.tagger.clusters, self.clusters)
         self.setItemExpanded(self.clusters, True)
-        self.connect(self.tagger, QtCore.SIGNAL("cluster_added"), self.add_cluster)
-        self.connect(self.tagger, QtCore.SIGNAL("cluster_removed"), self.remove_cluster)
+        self.tagger.cluster_added.connect(self.add_cluster)
+        self.tagger.cluster_removed.connect(self.remove_cluster)
 
     def remove_cluster(self, cluster):
-        index = self.clusters.indexOfChild(self.panel.item_from_object(cluster))
-        if self.clusters.takeChild(index):
-            for file in cluster.files:
-                self.panel.unregister_object(file)
-            self.panel.unregister_object(cluster)
+        cluster.item.setSelected(False)
+        self.clusters.removeChild(cluster.item)
 
 
 class AlbumTreeView(BaseTreeView):
 
     def __init__(self, window, parent=None):
         BaseTreeView.__init__(self, window, parent)
-        self.track_colors = {
-            File.NORMAL: self.config.setting["color_saved"],
-            File.CHANGED: self.palette().text().color(),
-            File.PENDING: self.config.setting["color_pending"],
-            File.ERROR: self.config.setting["color_error"],
-        }
-        self.connect(self.tagger, QtCore.SIGNAL("album_added"), self.add_album)
-        self.connect(self.tagger, QtCore.SIGNAL("album_removed"), self.remove_album)
-        self.connect(self.tagger, QtCore.SIGNAL("album_updated"), self.update_album)
-        self.connect(self.tagger, QtCore.SIGNAL("track_updated"), self.update_track)
-
-    def update_track(self, track, item=None, update_album=True):
-        if item is None:
-            try:
-                item = self.panel.item_from_object(track)
-            except KeyError:
-                self.log.debug("Item for %r not found", track)
-                return
-        if track.num_linked_files == 1:
-            file = track.linked_files[0]
-            color = self.track_colors[file.state]
-            icon = self.panel.decide_file_icon(file)
-
-            # remove old files
-            for i in range(item.childCount()):
-                file_item = item.takeChild(0)
-                self.panel.unregister_object(item=file_item)
-        else:
-            color = self.palette().text().color()
-            bgcolor = get_match_color(1, self.palette().base().color())
-            icon = self.panel.icon_note
-
-            #Add linked files (there will either be 0 or >1)
-            oldnum = item.childCount()
-            newnum = track.num_linked_files
-            # remove old items
-            if oldnum > newnum:
-                for i in range(oldnum - newnum):
-                    file_item = item.takeChild(newnum - 1)
-                    self.panel.unregister_object(item=file_item)
-                oldnum = newnum
-            # update existing items
-            file_item = None
-            for i in range(oldnum):
-                file_item = item.child(i)
-                file = track.linked_files[i]
-                self.panel.update_object(file, file_item)
-                self.panel.update_file(file, file_item)
-            # add new items
-            if newnum > oldnum:
-                for i in range(oldnum, newnum):
-                    file_item = NoSortTreeWidgetItem(item, file_item)
-                    file = track.linked_files[i]
-                    self.panel.register_object(file, file_item)
-                    self.panel.update_file(file, file_item)
-            self.expandItem(item)
-        item.setIcon(0, icon)
-        for i, column in enumerate(self.columns):
-            text, similarity = track.column(column[1])
-            item.setText(i, text)
-            item.setTextColor(i, color)
-            item.setBackgroundColor(i, get_match_color(similarity, self.palette().base().color()))
-        item.setData(1, QtCore.Qt.UserRole, QtCore.QVariant(track.metadata.length or 0))
-        if update_album:
-            self.update_album(track.album, update_tracks=False)
+        self.tagger.album_added.connect(self.add_album)
+        self.tagger.album_removed.connect(self.remove_album)
 
     def add_album(self, album):
-        item = SortTreeWidgetItem(self)
-        self.panel.register_object(album, item)
-        item.setIcon(0, self.panel.icon_cd)
-        for i, column in enumerate(self.columns):
+        item = AlbumItem(album, True, self)
+        item.setIcon(0, AlbumItem.icon_cd)
+        for i, column in enumerate(MainPanel.columns):
             font = item.font(i)
             font.setBold(True)
             item.setFont(i, font)
             item.setText(i, album.column(column[1]))
         self.add_cluster(album.unmatched_files, item)
 
-    def update_album(self, album, update_tracks=True):
-        try:
-            album_item = self.panel.item_from_object(album)
-        except KeyError:
-            self.log.debug("Item for %r not found", album)
-            return
+    def remove_album(self, album):
+        album.item.setSelected(False)
+        self.takeTopLevelItem(self.indexOfTopLevelItem(album.item))
+
+
+class TreeItem(QtGui.QTreeWidgetItem):
+
+    __lt__ = lambda self, other: False
+
+    def __init__(self, obj, sortable, *args):
+        QtGui.QTreeWidgetItem.__init__(self, *args)
+        self.obj = obj
+        if obj is not None:
+            obj.item = self
+        if sortable:
+            self.__lt__ = self._lt
+
+    def _lt(self, other):
+        column = self.treeWidget().sortColumn()
+        if column == 1:
+            return (self.obj.metadata.length or 0) < (other.obj.metadata.length or 0)
+        return self.text(column).toLower() < other.text(column).toLower()
+
+
+class ClusterItem(TreeItem):
+
+    def __init__(self, *args):
+        TreeItem.__init__(self, *args)
+        self.setIcon(0, ClusterItem.icon_dir)
+
+    def update(self):
+        for i, column in enumerate(MainPanel.columns):
+            self.setText(i, self.obj.column(column[1]))
+        album = self.obj.related_album
+        if self.obj.special and album and album.loaded:
+            album.item.update(update_tracks=False)
+        if self.isSelected():
+            TreeItem.window.update_selection()
+
+    def add_file(self, file):
+        self.add_files([file])
+
+    def add_files(self, files):
+        if self.obj.hide_if_empty and self.obj.files:
+            self.setHidden(False)
+        self.update()
+        items = []
+        for file in files:
+            item = FileItem(file, True)
+            item.update()
+            items.append(item)
+        self.addChildren(items)
+
+    def remove_file(self, file):
+        file.item.setSelected(False)
+        self.removeChild(file.item)
+        self.update()
+        if self.obj.hide_if_empty and not self.obj.files:
+            self.setHidden(True)
+
+
+class AlbumItem(TreeItem):
+
+    def update(self, update_tracks=True):
+        album = self.obj
         if update_tracks:
-            oldnum = album_item.childCount() - 1
+            oldnum = self.childCount() - 1
             newnum = len(album.tracks)
-            # remove old items
-            if oldnum > newnum:
-                for i in range(oldnum - newnum):
-                    item = album_item.takeChild(newnum - 1)
-                    self.panel.unregister_object(item=item)
+            if oldnum > newnum: # remove old items
+                for i in xrange(oldnum - newnum):
+                    self.takeChild(newnum - 1)
                 oldnum = newnum
             # update existing items
-            item = None
-            for i in range(oldnum):
-                item = album_item.child(i)
+            for i in xrange(oldnum):
+                item = self.child(i)
                 track = album.tracks[i]
-                self.panel.update_object(track, item)
-                self.update_track(track, item)
-            # add new items
-            if newnum > oldnum:
-                for i in range(oldnum, newnum):
-                    item = NoSortTreeWidgetItem(album_item, item)
+                item.obj = track
+                track.item = item
+                item.update(update_album=False)
+            if newnum > oldnum: # add new items
+                items = []
+                for i in xrange(newnum - 1, oldnum - 1, -1): # insertChildren is backwards
+                    item = TrackItem(album.tracks[i], False)
                     item.setHidden(False) # Workaround to make sure the parent state gets updated
-                    track = album.tracks[i]
-                    self.panel.register_object(track, item)
-                    self.update_track(track, item, update_album=False)
-        for i, column in enumerate(self.columns):
-            font = album_item.font(i)
-            if album.is_complete():
-                icon = self.panel.icon_cd_saved
+                    items.append(item)
+                self.insertChildren(oldnum, items)
+                for item in items: # Update after insertChildren so that setExpanded works
+                    item.update(update_album=False)
+        self.setIcon(0, AlbumItem.icon_cd_saved if album.is_complete() else AlbumItem.icon_cd)
+        for i, column in enumerate(MainPanel.columns):
+            self.setText(i, album.column(column[1]))
+        if self.isSelected():
+            TreeItem.window.update_selection()
+
+
+class TrackItem(TreeItem):
+
+    def update(self, update_album=True):
+        track = self.obj
+        if track.num_linked_files == 1:
+            file = track.linked_files[0]
+            file.item = None
+            color = TrackItem.track_colors[file.state]
+            bgcolor = get_match_color(file.similarity, TreeItem.base_color)
+            icon = FileItem.decide_file_icon(file)
+            self.takeChildren()
+        else:
+            color = TreeItem.text_color
+            bgcolor = get_match_color(1, TreeItem.base_color)
+            icon = TrackItem.icon_note
+            oldnum = self.childCount()
+            newnum = track.num_linked_files
+            if oldnum > newnum: # remove old items
+                for i in xrange(oldnum - newnum):
+                    self.takeChild(newnum - 1).obj.item = None
+                oldnum = newnum
+            for i in xrange(oldnum): # update existing items
+                item = self.child(i)
+                file = track.linked_files[i]
+                item.obj = file
+                file.item = item
+                item.update()
+            if newnum > oldnum: # add new items
+                items = []
+                for i in xrange(newnum - 1, oldnum - 1, -1):
+                    item = FileItem(track.linked_files[i], False)
+                    item.update()
+                    items.append(item)
+                self.addChildren(items)
+            self.setExpanded(True)
+        self.setIcon(0, icon)
+        for i, column in enumerate(MainPanel.columns):
+            self.setText(i, track.column(column[1]))
+            self.setForeground(i, color)
+            self.setBackground(i, bgcolor)
+        if self.isSelected():
+            TreeItem.window.update_selection()
+        if update_album:
+            self.parent().update(update_tracks=False)
+
+
+class FileItem(TreeItem):
+
+    def update(self):
+        file = self.obj
+        self.setIcon(0, FileItem.decide_file_icon(file))
+        color = FileItem.file_colors[file.state]
+        bgcolor = get_match_color(file.similarity, TreeItem.base_color)
+        for i, column in enumerate(MainPanel.columns):
+            self.setText(i, file.column(column[1]))
+            self.setForeground(i, color)
+            self.setBackground(i, bgcolor)
+        if self.isSelected():
+            TreeItem.window.update_selection()
+
+    @staticmethod
+    def decide_file_icon(file):
+        if file.state == File.ERROR:
+            return FileItem.icon_error
+        elif isinstance(file.parent, Track):
+            if file.state == File.NORMAL:
+                return FileItem.icon_saved
+            elif file.state == File.PENDING:
+                return FileItem.match_pending_icons[int(file.similarity * 5 + 0.5)]
             else:
-                icon = self.panel.icon_cd
-            album_item.setIcon(0, icon)
-            album_item.setFont(i, font)
-            album_item.setText(i, album.column(column[1]))
-        album_item.setData(1, QtCore.Qt.UserRole, QtCore.QVariant(album.metadata.length or 0))
-        if album_item.isSelected():
-            self.window.updateSelection(self.panel.selected_objects())
-
-    def remove_album(self, album):
-        index = self.indexOfTopLevelItem(self.panel.item_from_object(album))
-        if self.takeTopLevelItem(index):
-            for track in album.tracks:
-                self.panel.unregister_object(track)
-            self.panel.unregister_object(album)
-            if album == self.tagger.nats:
-                self.tagger.nats = None
-
+                return FileItem.match_icons[int(file.similarity * 5 + 0.5)]
+        elif file.state == File.PENDING:
+            return FileItem.icon_file_pending
+        else:
+            return FileItem.icon_file
