@@ -17,12 +17,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from PyQt4 import QtCore, QtGui, QtNetwork
+from PyQt4 import QtCore, QtGui
 
-import os.path
 import sys
+import os.path
 
-from picard import __version__
 from picard.album import Album
 from picard.file import File
 from picard.track import Track
@@ -35,11 +34,12 @@ from picard.ui.metadatabox import MetadataBox
 from picard.ui.filebrowser import FileBrowser
 from picard.ui.tagsfromfilenames import TagsFromFileNamesDialog
 from picard.ui.options.dialog import OptionsDialog
-from picard.ui.tageditor import TagEditor
+from picard.ui.infodialog import InfoDialog
 from picard.ui.passworddialog import PasswordDialog
 from picard.util import icontheme, webbrowser2, find_existing_path
 from picard.util.cdrom import get_cdrom_drives
 from picard.plugin import ExtensionPoint
+from picard import __version__
 import picard.musicdns
 
 ui_init = ExtensionPoint()
@@ -55,6 +55,8 @@ class MainWindow(QtGui.QMainWindow):
                QtCore.QVariant.toPoint),
         Option("persist", "window_size", QtCore.QSize(780, 560),
                QtCore.QVariant.toSize),
+        Option("persist", "bottom_splitter_state", QtCore.QByteArray(),
+               QtCore.QVariant.toByteArray),
         BoolOption("persist", "window_maximized", False),
         BoolOption("persist", "view_cover_art", False),
         BoolOption("persist", "view_file_browser", False),
@@ -64,6 +66,7 @@ class MainWindow(QtGui.QMainWindow):
     def __init__(self, parent=None):
         QtGui.QMainWindow.__init__(self, parent)
         self.selected_objects = []
+        self.ignore_selection_changes = False
         self.setupUi()
 
     def setupUi(self):
@@ -82,35 +85,33 @@ class MainWindow(QtGui.QMainWindow):
         self.create_toolbar()
         self.create_menus()
 
-        centralWidget = QtGui.QWidget(self)
-        self.setCentralWidget(centralWidget)
+        mainLayout = QtGui.QSplitter(QtCore.Qt.Vertical)
+        mainLayout.setContentsMargins(0, 0, 0, 0)
+        mainLayout.setHandleWidth(1)
 
-        self.panel = MainPanel(self, centralWidget)
+        self.panel = MainPanel(self, mainLayout)
         self.file_browser = FileBrowser(self.panel)
         if not self.show_file_browser_action.isChecked():
             self.file_browser.hide()
         self.panel.insertWidget(0, self.file_browser)
         self.panel.restore_state()
 
-        self.orig_metadata_box = MetadataBox(self, _("Original Metadata"), True)
-        self.orig_metadata_box.disable()
-        self.metadata_box = MetadataBox(self, _("New Metadata"), False)
-        self.metadata_box.disable()
-
+        self.metadata_box = MetadataBox(self)
         self.cover_art_box = CoverArtBox(self)
         if not self.show_cover_art_action.isChecked():
             self.cover_art_box.hide()
 
         bottomLayout = QtGui.QHBoxLayout()
-        bottomLayout.addWidget(self.orig_metadata_box, 1)
+        bottomLayout.setContentsMargins(0, 0, 0, 0)
+        bottomLayout.setSpacing(0)
         bottomLayout.addWidget(self.metadata_box, 1)
         bottomLayout.addWidget(self.cover_art_box, 0)
+        bottom = QtGui.QWidget()
+        bottom.setLayout(bottomLayout)
 
-        mainLayout = QtGui.QVBoxLayout()
-        mainLayout.addWidget(self.panel, 1)
-        mainLayout.addLayout(bottomLayout, 0)
-
-        centralWidget.setLayout(mainLayout)
+        mainLayout.addWidget(self.panel)
+        mainLayout.addWidget(bottom)
+        self.setCentralWidget(mainLayout)
 
         # FIXME: use QApplication's clipboard
         self._clipboard = []
@@ -118,12 +119,52 @@ class MainWindow(QtGui.QMainWindow):
         for function in ui_init:
             function(self)
 
+    def keyPressEvent(self, event):
+        if event.matches(QtGui.QKeySequence.Delete):
+            if self.metadata_box.hasFocus():
+                self.metadata_box.remove_selected_tag()
+            else:
+                self.remove()
+        else:
+            QtGui.QMainWindow.keyPressEvent(self, event)
+
+    def show(self):
         self.restoreWindowState()
+        QtGui.QMainWindow.show(self)
+        self.metadata_box.restore_state()
 
     def closeEvent(self, event):
+        if self.config.setting["quit_confirmation"] and not self.show_quit_confirmation():
+            event.ignore()
+            return
         self.saveWindowState()
         event.accept()
 
+    def show_quit_confirmation(self):
+        unsaved_files = sum(a.get_num_unsaved_files() for a in self.tagger.albums.itervalues())
+        QMessageBox = QtGui.QMessageBox
+
+        if unsaved_files > 0:
+            msg = QMessageBox(self)
+            msg.setIcon(QMessageBox.Question)
+            msg.setWindowModality(QtCore.Qt.WindowModal)
+            msg.setWindowTitle(_(u"Unsaved Changes"))
+            msg.setText(_(u"Are you sure you want to quit Picard?"))
+            txt = ungettext(
+                "There is %d unsaved file. Closing Picard will lose all unsaved changes.",
+                "There are %d unsaved files. Closing Picard will lose all unsaved changes.",
+                unsaved_files) % unsaved_files
+            msg.setInformativeText(txt)
+            cancel = msg.addButton(QMessageBox.Cancel)
+            msg.setDefaultButton(cancel)
+            msg.addButton(_(u"&Quit Picard"), QMessageBox.YesRole)
+            ret = msg.exec_()
+
+            if ret == QMessageBox.Cancel:
+                return False
+
+        return True
+
     def saveWindowState(self):
         self.config.persist["window_state"] = self.saveState()
         isMaximized = int(self.windowState()) & QtCore.Qt.WindowMaximized != 0
@@ -140,8 +181,10 @@ class MainWindow(QtGui.QMainWindow):
         self.config.persist["window_maximized"] = isMaximized
         self.config.persist["view_cover_art"] = self.show_cover_art_action.isChecked()
         self.config.persist["view_file_browser"] = self.show_file_browser_action.isChecked()
+        self.config.persist["bottom_splitter_state"] = self.centralWidget().saveState()
         self.file_browser.save_state()
         self.panel.save_state()
+        self.metadata_box.save_state()
 
     def restoreWindowState(self):
         self.restoreState(self.config.persist["window_state"])
@@ -155,25 +198,41 @@ class MainWindow(QtGui.QMainWindow):
         self.resize(size)
         if self.config.persist["window_maximized"]:
             self.setWindowState(QtCore.Qt.WindowMaximized)
+        bottom_splitter_state = self.config.persist["bottom_splitter_state"]
+        if bottom_splitter_state.isEmpty():
+            self.centralWidget().setSizes([366, 194])
+        else:
+            self.centralWidget().restoreState(bottom_splitter_state)
         self.file_browser.restore_state()
 
     def create_statusbar(self):
         """Creates a new status bar."""
         self.statusBar().showMessage("Ready")
         self.file_counts_label = QtGui.QLabel()
+        self.listening_label = QtGui.QLabel()
+        self.listening_label.setVisible(False)
+        self.listening_label.setToolTip(_("Picard listens on a port to integrate with your browser and downloads release"
+                                          " information when you click the \"Tagger\" buttons on the MusicBrainz website"))
         self.statusBar().addPermanentWidget(self.file_counts_label)
-        self.connect(self.tagger, QtCore.SIGNAL("file_state_changed"), self.update_statusbar)
-        self.update_statusbar(0)
+        self.statusBar().addPermanentWidget(self.listening_label)
+        self.connect(self.tagger, QtCore.SIGNAL("file_state_changed"), self.update_statusbar_files)
+        self.connect(self.tagger, QtCore.SIGNAL("listen_port_changed"), self.update_statusbar_listen_port)
+        self.update_statusbar_files(0)
 
-    def update_statusbar(self, num_pending_files):
+    def update_statusbar_files(self, num_pending_files):
         """Updates the status bar information."""
         self.file_counts_label.setText(_(" Files: %(files)d, Pending Files: %(pending)d ")
             % {"files": self.tagger.num_files(), "pending": num_pending_files})
 
+    def update_statusbar_listen_port(self, listen_port):
+        self.listening_label.setVisible(True)
+        self.listening_label.setText(_(" Listening on port %(port)d ") % {"port": listen_port})
+
     def set_statusbar_message(self, message, *args, **kwargs):
         """Set the status bar message."""
         try:
-            self.log.debug(repr(message.replace('%%s', '%%r')), *args)
+            if message:
+                self.log.debug(repr(message.replace('%%s', '%%r')), *args)
         except:
             pass
         self.tagger.thread_pool.call_from_thread(
@@ -191,8 +250,18 @@ class MainWindow(QtGui.QMainWindow):
         """Set the status bar message."""
         self.statusBar().clearMessage()
 
-    def create_actions(self):
+    def _on_submit(self):
+        if self.tagger.use_acoustid:
+            if not self.config.setting["acoustid_apikey"]:
+                QtGui.QMessageBox.warning(self,
+                    _(u"Submission Error"),
+                    _(u"You need to configure your AcoustID API key before you can submit fingerprints."))
+            else:
+                self.tagger.acoustidmanager.submit()
+        else:
+            self.tagger.puidmanager.submit()
 
+    def create_actions(self):
         self.options_action = QtGui.QAction(icontheme.lookup('preferences-desktop'), _("&Options..."), self)
         self.connect(self.options_action, QtCore.SIGNAL("triggered()"), self.show_options)
 
@@ -243,10 +312,10 @@ class MainWindow(QtGui.QMainWindow):
         self.save_action.setEnabled(False)
         self.connect(self.save_action, QtCore.SIGNAL("triggered()"), self.save)
 
-        self.submit_action = QtGui.QAction(icontheme.lookup('picard-submit'), _(u"S&ubmit PUIDs"), self)
-        self.submit_action.setStatusTip(_(u"Submit PUIDs to MusicBrainz"))
+        self.submit_action = QtGui.QAction(icontheme.lookup('picard-submit'), _(u"S&ubmit"), self)
+        self.submit_action.setStatusTip(_(u"Submit fingerprints"))
         self.submit_action.setEnabled(False)
-        self.connect(self.submit_action, QtCore.SIGNAL("triggered()"), self.tagger.puidmanager.submit)
+        self.connect(self.submit_action, QtCore.SIGNAL("triggered()"), self._on_submit)
 
         self.exit_action = QtGui.QAction(_(u"E&xit"), self)
         # TR: Keyboard shortcut for "Exit"
@@ -255,10 +324,14 @@ class MainWindow(QtGui.QMainWindow):
 
         self.remove_action = QtGui.QAction(icontheme.lookup('list-remove'), _(u"&Remove"), self)
         self.remove_action.setStatusTip(_(u"Remove selected files/albums"))
-        self.remove_action.setShortcut(QtGui.QKeySequence.Delete)
         self.remove_action.setEnabled(False)
         self.connect(self.remove_action, QtCore.SIGNAL("triggered()"), self.remove)
 
+        self.browser_lookup_action = QtGui.QAction(icontheme.lookup('lookup-musicbrainz'), _(u"Lookup in &Browser"), self)
+        self.browser_lookup_action.setStatusTip(_(u"Lookup selected item on MusicBrainz website"))
+        self.browser_lookup_action.setEnabled(False)
+        self.browser_lookup_action.triggered.connect(self.browser_lookup)
+
         self.show_file_browser_action = QtGui.QAction(_(u"File &Browser"), self)
         self.show_file_browser_action.setCheckable(True)
         if self.config.persist["view_file_browser"]:
@@ -302,18 +375,16 @@ class MainWindow(QtGui.QMainWindow):
         self.autotag_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+L")))
         self.connect(self.autotag_action, QtCore.SIGNAL("triggered()"), self.autotag)
 
-        self.edit_tags_action = QtGui.QAction(icontheme.lookup('picard-edit-tags'), _(u"&Details..."), self)
-        self.edit_tags_action.setEnabled(False)
-        # TR: Keyboard shortcut for "Details"
-        self.edit_tags_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+I")))
-        self.connect(self.edit_tags_action, QtCore.SIGNAL("triggered()"), self.edit_tags)
+        self.view_info_action = QtGui.QAction(icontheme.lookup('picard-edit-tags'), _(u"&Info..."), self)
+        self.view_info_action.setEnabled(False)
+        # TR: Keyboard shortcut for "Info"
+        self.view_info_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+I")))
+        self.connect(self.view_info_action, QtCore.SIGNAL("triggered()"), self.view_info)
 
         self.refresh_action = QtGui.QAction(icontheme.lookup('view-refresh', icontheme.ICON_SIZE_MENU), _("&Refresh"), self)
+        self.refresh_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+R")))
         self.connect(self.refresh_action, QtCore.SIGNAL("triggered()"), self.refresh)
 
-        self.generate_playlist_action = QtGui.QAction(_("Generate &Playlist..."), self)
-        self.connect(self.generate_playlist_action, QtCore.SIGNAL("triggered()"), self.generate_playlist)
-
         self.enable_renaming_action = QtGui.QAction(_(u"&Rename Files"), self)
         self.enable_renaming_action.setCheckable(True)
         self.enable_renaming_action.setChecked(self.config.setting["rename_files"])
@@ -338,6 +409,14 @@ class MainWindow(QtGui.QMainWindow):
         self.connect(self.tagger.xmlws, QtCore.SIGNAL("authentication_required"), self.show_password_dialog)
         self.connect(self.tagger.xmlws, QtCore.SIGNAL("proxyAuthentication_required"), self.show_proxy_dialog)
 
+        self.open_file_action = QtGui.QAction(_(u"&Open..."), self)
+        self.open_file_action.setStatusTip(_(u"Open the file"))
+        self.connect(self.open_file_action, QtCore.SIGNAL("triggered()"), self.open_file)
+
+        self.open_folder_action = QtGui.QAction(_(u"Open &Folder..."), self)
+        self.open_folder_action.setStatusTip(_(u"Open the containing folder"))
+        self.connect(self.open_folder_action, QtCore.SIGNAL("triggered()"), self.open_folder)
+
     def toggle_rename_files(self, checked):
         self.config.setting["rename_files"] = checked
 
@@ -368,7 +447,7 @@ class MainWindow(QtGui.QMainWindow):
         menu.addAction(self.cut_action)
         menu.addAction(self.paste_action)
         menu.addSeparator()
-        menu.addAction(self.edit_tags_action)
+        menu.addAction(self.view_info_action)
         menu.addAction(self.remove_action)
         menu = self.menuBar().addMenu(_(u"&View"))
         menu.addAction(self.show_file_browser_action)
@@ -376,6 +455,8 @@ class MainWindow(QtGui.QMainWindow):
         menu.addSeparator()
         menu.addAction(self.toolbar.toggleViewAction())
         menu.addAction(self.search_toolbar.toggleViewAction())
+        menu.addSeparator()
+        menu.addAction(self.refresh_action)
         menu = self.menuBar().addMenu(_(u"&Options"))
         menu.addAction(self.enable_renaming_action)
         menu.addAction(self.enable_moving_action)
@@ -387,8 +468,8 @@ class MainWindow(QtGui.QMainWindow):
         menu.addAction(self.autotag_action)
         menu.addAction(self.analyze_action)
         menu.addAction(self.cluster_action)
+        menu.addAction(self.browser_lookup_action)
         menu.addSeparator()
-        #menu.addAction(self.generate_playlist_action)
         menu.addAction(self.tags_from_filenames_action)
         self.menuBar().addSeparator()
         menu = self.menuBar().addMenu(_(u"&Help"))
@@ -433,8 +514,9 @@ class MainWindow(QtGui.QMainWindow):
         toolbar.addAction(self.cluster_action)
         toolbar.addAction(self.autotag_action)
         toolbar.addAction(self.analyze_action)
-        toolbar.addAction(self.edit_tags_action)
+        toolbar.addAction(self.view_info_action)
         toolbar.addAction(self.remove_action)
+        toolbar.addAction(self.browser_lookup_action)
         self.search_toolbar = toolbar = self.addToolBar(_(u"&Search Bar"))
         toolbar.setObjectName("search_toolbar")
         search_panel = QtGui.QWidget(toolbar)
@@ -502,6 +584,8 @@ class MainWindow(QtGui.QMainWindow):
             # Use a custom file selection dialog to allow the selection of multiple directories
             file_dialog = QtGui.QFileDialog(self, "", current_directory)
             file_dialog.setFileMode(QtGui.QFileDialog.DirectoryOnly)
+            if sys.platform == "darwin": # The native dialog doesn't allow selecting >1 directory
+                file_dialog.setOption(QtGui.QFileDialog.DontUseNativeDialog)
             tree_view = file_dialog.findChild(QtGui.QTreeView)
             tree_view.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
             list_view = file_dialog.findChild(QtGui.QListView, "listView")
@@ -520,21 +604,6 @@ class MainWindow(QtGui.QMainWindow):
             directory = unicode(directory)
             self.tagger.add_directory(directory)
 
-    def generate_playlist(self):
-        """Generate a playlist."""
-        from picard.playlist import Playlist
-        currentDirectory = self.config.persist["current_directory"]
-        formats = [_(f[0]) for f in Playlist.formats]
-        selected_format = QtCore.QString()
-        filename = QtGui.QFileDialog.getSaveFileName(self, "", currentDirectory, ";;".join(formats), selected_format)
-        if filename:
-            filename = unicode(filename)
-            self.config.persist["current_directory"] = os.path.dirname(filename)
-            self.set_statusbar_message(_("Saving playlist %s...") % filename)
-            playlist = Playlist(self.selected_objects[0])
-            playlist.save(filename, formats.index(unicode(selected_format)))
-            self.set_statusbar_message(_("Playlist %s saved") % filename, 1000)
-
     def show_about(self):
         self.show_options("about")
 
@@ -543,38 +612,33 @@ class MainWindow(QtGui.QMainWindow):
         dialog.exec_()
 
     def show_help(self):
-        webbrowser2.open("http://musicbrainz.org/doc/PicardDocumentation")
+        webbrowser2.open("http://musicbrainz.org/doc/Picard_Documentation")
 
     def show_log(self):
         from picard.ui.logview import LogView
         w = LogView(self)
         w.show()
 
+    def confirm_va_removal(self):
+        return QtGui.QMessageBox.question(self,
+                _("Various Artists file naming scheme removal"),
+_("""The separate file naming scheme for various artists albums has been
+removed in this version of Picard. You currently do not use the this option,
+but have a separate file naming scheme defined. Do you want to remove it or
+merge it with your file naming scheme for single artist albums?"""),
+            _("Merge"), _("Remove"))
+
+    def show_va_removal_notice(self):
+        QtGui.QMessageBox.information(self,
+            _("Various Artists file naming scheme removal"),
+_("""The separate file naming scheme for various artists albums has been
+removed in this version of Picard. Your file naming scheme has automatically
+been merged with that of single artist albums."""),
+            QtGui.QMessageBox.Ok)
+
+
     def open_bug_report(self):
-        args = [
-            "component=Picard+Tagger",
-            "version=Picard+" + __version__,
-        ]
-        if sys.platform == "linux2":
-            args.append("os=Linux")
-        elif sys.platform == "win32":
-            import platform
-            release = platform.release()
-            if release in ("post2008Server", "7"):
-                args.append("os=Windows+7")
-            elif release in ("Vista", "2008Server"):
-                args.append("os=Windows+Vista")
-            elif release in ("XP", "2003Server"):
-                args.append("os=Windows+XP")
-            elif release in ("NT", "2000"):
-                args.append("os=Windows+NT%2F2000")
-            elif release in ("95", "98", "Me"):
-                args.append("os=Windows+95%2F98%2FMe")
-            else:
-                args.append("os=Windows+XP")
-        elif sys.platform == "darwin":
-            args.append("os=Mac+OS+X")
-        webbrowser2.open("http://bugs.musicbrainz.org/newticket?" + "&".join(args))
+        webbrowser2.open("http://musicbrainz.org/doc/Picard_Troubleshooting")
 
     def open_support_forum(self):
         webbrowser2.open("http://forums.musicbrainz.org/viewforum.php?id=2")
@@ -584,60 +648,94 @@ class MainWindow(QtGui.QMainWindow):
 
     def save(self):
         """Tell the tagger to save the selected objects."""
-        self.tagger.save(self.panel.selected_objects())
+        self.tagger.save(self.selected_objects)
 
     def remove(self):
         """Tell the tagger to remove the selected objects."""
-        self.tagger.remove(self.panel.selected_objects())
+        self.panel._ignore_selection_changes = True
+        self.tagger.remove(self.selected_objects)
+        self.panel._ignore_selection_changes = False
+        self.panel.update_current_view()
 
     def analyze(self):
-        self.tagger.analyze(self.panel.selected_objects())
+        if not self.config.setting['enable_fingerprinting']:
+            if self.show_analyze_settings_info():
+                self.show_options("fingerprinting")
+            if not self.config.setting['enable_fingerprinting']:
+                return
+        return self.tagger.analyze(self.selected_objects)
 
-    def edit_tags(self, objs=None):
-        if not objs:
-            objs = self.selected_objects
-        objs = self.tagger.get_files_from_objects(objs)
-        dialog = TagEditor(objs, self)
+    def open_file(self):
+        files = self.tagger.get_files_from_objects(self.selected_objects)
+        for file in files:
+            url = QtCore.QUrl.fromLocalFile(file.filename)
+            QtGui.QDesktopServices.openUrl(url)
+
+    def open_folder(self):
+        files = self.tagger.get_files_from_objects(self.selected_objects)
+        for file in files:
+            url = QtCore.QUrl.fromLocalFile(os.path.dirname(file.filename))
+            QtGui.QDesktopServices.openUrl(url)
+
+    def show_analyze_settings_info(self):
+        ret = QtGui.QMessageBox.question(self,
+            _(u"Configuration Required"),
+            _(u"Audio fingerprinting is not yet configured. Would you like to configure it now?"),
+            QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
+            QtGui.QMessageBox.Yes)
+        return ret == QtGui.QMessageBox.Yes
+
+    def view_info(self):
+        file = self.tagger.get_files_from_objects(self.selected_objects)[0]
+        dialog = InfoDialog(file, self)
         dialog.exec_()
 
     def cluster(self):
-        self.tagger.cluster(self.panel.selected_objects())
+        self.tagger.cluster(self.selected_objects)
 
     def refresh(self):
-        self.tagger.refresh(self.panel.selected_objects())
+        self.tagger.refresh(self.selected_objects)
+
+    def browser_lookup(self):
+        self.tagger.browser_lookup(self.selected_objects[0])
 
     def update_actions(self):
         can_remove = False
         can_save = False
-        can_edit_tags = False
         can_analyze = False
         can_refresh = False
         can_autotag = False
+        single = self.selected_objects[0] if len(self.selected_objects) == 1 else None
+        can_view_info = bool(single and single.can_view_info())
+        can_browser_lookup = bool(single and single.can_browser_lookup())
         for obj in self.selected_objects:
-            if picard.musicdns.ofa and obj.can_analyze():
+            if obj is None:
+                continue
+            if obj.can_analyze():
                 can_analyze = True
             if obj.can_save():
                 can_save = True
             if obj.can_remove():
                 can_remove = True
-            if obj.can_edit_tags():
-                can_edit_tags = True
             if obj.can_refresh():
                 can_refresh = True
             if obj.can_autotag():
                 can_autotag = True
-            if can_save and can_remove and can_edit_tags and can_refresh \
-                    and can_autotag:
+            if can_save and can_remove and can_refresh and can_autotag:
                 break
         self.remove_action.setEnabled(can_remove)
         self.save_action.setEnabled(can_save)
-        self.edit_tags_action.setEnabled(can_edit_tags)
+        self.view_info_action.setEnabled(can_view_info)
         self.analyze_action.setEnabled(can_analyze)
         self.refresh_action.setEnabled(can_refresh)
         self.autotag_action.setEnabled(can_autotag)
+        self.browser_lookup_action.setEnabled(can_browser_lookup)
         self.cut_action.setEnabled(bool(self.selected_objects))
 
-    def updateSelection(self, objects=None):
+    def update_selection(self, objects=None):
+        if self.ignore_selection_changes:
+            return
+
         if objects is not None:
             self.selected_objects = objects
         else:
@@ -645,49 +743,37 @@ class MainWindow(QtGui.QMainWindow):
 
         self.update_actions()
 
-        orig_metadata = None
         metadata = None
-        is_album = False
-        statusBar = u""
-        file = None
+        statusbar = u""
+        obj = None
+
         if len(objects) == 1:
-            obj = objects[0]
+            obj = list(objects)[0]
             if isinstance(obj, File):
-                orig_metadata = obj.orig_metadata
                 metadata = obj.metadata
-                statusBar = obj.filename
+                statusbar = obj.filename
                 if obj.state == obj.ERROR:
-                    statusBar += _(" (Error: %s)") % obj.error
-                file = obj
+                    statusbar += _(" (Error: %s)") % obj.error
             elif isinstance(obj, Track):
+                metadata = obj.metadata
                 if obj.num_linked_files == 1:
                     file = obj.linked_files[0]
-                    orig_metadata = file.orig_metadata
-                    metadata = file.metadata
-                    statusBar = "%s (%d%%)" % (file.filename, file.similarity * 100)
-                    if file.state == file.ERROR:
-                        statusBar += _(" (Error: %s)") % file.error
-                elif obj.num_linked_files == 0:
-                    metadata = obj.metadata
-                else:
-                    metadata = obj.metadata
-                    #Show dup zaper
-            elif isinstance(obj, Cluster):
-                orig_metadata = obj.metadata
-                is_album = True
-            elif isinstance(obj, Album):
+                    statusbar = "%s (%d%%)" % (file.filename, file.similarity * 100)
+                    if file.state == File.ERROR:
+                        statusbar += _(" (Error: %s)") % file.error
+            elif obj.can_edit_tags():
                 metadata = obj.metadata
-                is_album = True
 
-        self.orig_metadata_box.set_metadata(orig_metadata, is_album)
-        self.metadata_box.set_metadata(metadata, is_album, file=file)
-        self.cover_art_box.set_metadata(metadata)
-        self.set_statusbar_message(statusBar)
+        self.metadata_box.update_selection()
+        self.metadata_box.update()
+        self.cover_art_box.set_metadata(metadata, obj)
+        self.set_statusbar_message(statusbar)
 
     def show_cover_art(self):
         """Show/hide the cover art box."""
         if self.show_cover_art_action.isChecked():
             self.cover_art_box.show()
+            self.metadata_box.shrink_columns()
         else:
             self.cover_art_box.hide()
 
@@ -711,14 +797,14 @@ class MainWindow(QtGui.QMainWindow):
         dialog.exec_()
 
     def autotag(self):
-        self.tagger.autotag(self.panel.selected_objects())
+        self.tagger.autotag(self.selected_objects)
 
     def cut(self):
-        self._clipboard = self.panel.selected_objects()
+        self._clipboard = self.selected_objects
         self.paste_action.setEnabled(bool(self._clipboard))
 
     def paste(self):
-        selected_objects = self.panel.selected_objects()
+        selected_objects = self.selected_objects
         if not selected_objects:
             target = self.tagger.unmatched_files
         else:
