@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-tag_names = {
+TAG_NAMES = {
     'album': N_('Album'),
     'artist': N_('Artist'),
     'title': N_('Title'),
@@ -39,7 +39,9 @@ tag_names = {
     'mood': N_('Mood'),
     'bpm': N_('BPM'),
     'copyright': N_('Copyright'),
+    'license': N_('License'),
     'composer': N_('Composer'),
+    'writer': N_('Writer'),
     'conductor': N_('Conductor'),
     'lyricist': N_('Lyricist'),
     'arranger': N_('Arranger'),
@@ -48,14 +50,17 @@ tag_names = {
     'subtitle': N_('Subtitle'),
     'discsubtitle': N_('Disc Subtitle'),
     'remixer': N_('Remixer'),
-    'musicbrainz_trackid': N_('MusicBrainz Track Id'),
+    'musicbrainz_trackid': N_('MusicBrainz Recording Id'),
     'musicbrainz_albumid': N_('MusicBrainz Release Id'),
     'musicbrainz_artistid': N_('MusicBrainz Artist Id'),
     'musicbrainz_albumartistid': N_('MusicBrainz Release Artist Id'),
+    'musicbrainz_workid': N_('MusicBrainz Work Id'),
     'musicbrainz_discid': N_('MusicBrainz Disc Id'),
     'musicbrainz_sortname': N_('MusicBrainz Sort Name'),
     'musicip_puid': N_('MusicIP PUID'),
     'musicip_fingerprint': N_('MusicIP Fingerprint'),
+    'acoustid_id': N_('AcoustID'),
+    'acoustid_fingerprint': N_('AcoustID Fingerprint'),
     'discid': N_('Disc Id'),
     'website': N_('Website'),
     'compilation': N_('Compilation'),
@@ -76,20 +81,23 @@ tag_names = {
     'mixer': N_('Mixer'),
     'language': N_('Language'),
     'script': N_('Script'),
+    '~length': N_('Length'),
+    '~rating': N_('Rating'),
 }
 
 def display_tag_name(name):
     if ':' in name:
         name, desc = name.split(':', 1)
-        name = _(tag_names.get(name + ':', name))
+        name = _(TAG_NAMES.get(name + ':', name))
         return '%s [%s]' % (_(name), desc)
     else:
-        new_name = tag_names.get(name)
+        new_name = TAG_NAMES.get(name)
         if new_name is None:
-            new_name = tag_names.get(name + ':')
+            new_name = TAG_NAMES.get(name + ':')
             if new_name is None:
                 return _(name)
             else:
                 return '%s []' % (_(new_name),)
         else:
             return _(new_name)
+
