@@ -27,14 +27,17 @@ import os
 import sys
 import re
 import traceback
+import time
 from collections import deque, defaultdict
 from PyQt4 import QtCore, QtNetwork, QtXml
+from PyQt4.QtCore import QUrl
 from picard import version_string
 from picard.util import partial
-from picard.const import PUID_SUBMIT_HOST, PUID_SUBMIT_PORT
+from picard.const import PUID_SUBMIT_HOST, PUID_SUBMIT_PORT, ACOUSTID_KEY
 
 
 REQUEST_DELAY = defaultdict(lambda: 1000)
+REQUEST_DELAY[('api.acoustid.org', 80)] = 333
 USER_AGENT_STRING = 'MusicBrainz%%20Picard-%s' % version_string
 
 
@@ -57,6 +60,12 @@ class XmlNode(object):
     def __repr__(self):
         return repr(self.__dict__)
 
+    def append_child(self, name, node=None):
+        if node is None:
+            node = XmlNode()
+        self.children.setdefault(name, []).append(node)
+        return node
+
     def __getattr__(self, name):
         try:
             return self.children[name]
@@ -80,7 +89,7 @@ class XmlHandler(QtXml.QXmlDefaultHandler):
         node = XmlNode()
         for i in xrange(attrs.count()):
             node.attribs[self._node_name(attrs.localName(i))] = unicode(attrs.value(i))
-        self.node.children.setdefault(self._node_name(name), []).append(node)
+        self.node.append_child(self._node_name(name), node)
         self.path.append(self.node)
         self.node = node
         return True
@@ -134,21 +143,34 @@ class XmlWebService(QtCore.QObject):
 
     def _start_request(self, method, host, port, path, data, handler, xml, mblogin=False):
         self.log.debug("%s http://%s:%d%s", method, host, port, path)
-        url = QtCore.QUrl.fromEncoded("http://%s:%d%s" % (host, port, path))
+        url = QUrl.fromEncoded("http://%s:%d%s" % (host, port, path))
         if mblogin:
             url.setUserName(self.config.setting["username"])
             url.setPassword(self.config.setting["password"])
         request = QtNetwork.QNetworkRequest(url)
         request.setRawHeader("User-Agent", "MusicBrainz-Picard/%s" % version_string)
-        if method == "POST" and host == self.config.setting["server_host"]:
-            request.setHeader(QtNetwork.QNetworkRequest.ContentTypeHeader, "application/xml; charset=utf-8")
+        if data is not None:
+            if method == "POST" and host == self.config.setting["server_host"]:
+                request.setHeader(QtNetwork.QNetworkRequest.ContentTypeHeader, "application/xml; charset=utf-8")
+            else:
+                request.setHeader(QtNetwork.QNetworkRequest.ContentTypeHeader, "application/x-www-form-urlencoded")
         send = self._request_methods[method]
         reply = send(request, data) if data is not None else send(request)
         key = (host, port)
-        self._last_request_times[key] = QtCore.QTime.currentTime()
+        self._last_request_times[key] = time.time()
         self._active_requests[reply] = (request, handler, xml)
         return True
 
+    @staticmethod
+    def urls_equivalent(leftUrl, rightUrl):
+        """
+            Lazy method to determine whether two QUrls are equivalent. At the moment it assumes that if ports are unset
+            that they are port 80 - in absence of a URL normalization function in QUrl or ability to use qHash
+            from QT 4.7
+        """
+        return leftUrl.port(80) == rightUrl.port(80) and \
+            leftUrl.toString(QUrl.FormattingOption(QUrl.RemovePort)) == rightUrl.toString(QUrl.FormattingOption(QUrl.RemovePort))
+
     def _process_reply(self, reply):
         try:
             request, handler, xml = self._active_requests.pop(reply)
@@ -156,13 +178,42 @@ class XmlWebService(QtCore.QObject):
             self.log.error("Error: Request not found for %s" % str(reply.request().url().toString()))
             return
         error = int(reply.error())
+        redirect = reply.attribute(QtNetwork.QNetworkRequest.RedirectionTargetAttribute).toUrl()
+        self.log.debug("Received reply for %s: HTTP %d (%s)",
+                       reply.request().url().toString(),
+                       reply.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute).toInt()[0],
+                       reply.attribute(QtNetwork.QNetworkRequest.HttpReasonPhraseAttribute).toString())
         if handler is not None:
             if error:
-                #print "ERROR", reply.error(), reply.errorString()
-                #for name in reply.rawHeaderList():
-                #    print name, reply.rawHeader(name)
-                self.log.debug("HTTP Error: %d", error)
-            if xml:
+                self.log.error("Network request error for %s: %s (QT code %d, HTTP code %d)",
+                              reply.request().url().toString(),
+                              reply.errorString(),
+                              error,
+                              reply.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute).toInt()[0])
+
+            # Redirect if found and not infinite
+            if not redirect.isEmpty() and not XmlWebService.urls_equivalent(redirect, reply.request().url()):
+                self.log.debug("Redirect to %s requested", redirect.toString())
+                redirect_host = str(redirect.host())
+                redirect_port = redirect.port(80)
+
+                url = request.url()
+                original_host = str(url.host())
+                original_port = url.port(80)
+
+                if (original_host, original_port) in REQUEST_DELAY:
+                    self.log.debug("Setting rate limit for %s:%i to %i" %
+                            (redirect_host, redirect_port,
+                            REQUEST_DELAY[(original_host, original_port)]))
+                    REQUEST_DELAY[(redirect_host, redirect_port)] =\
+                        REQUEST_DELAY[(original_host, original_port)]
+
+                self.get(redirect_host,
+                         redirect_port,
+                         # retain path, query string and anchors from redirect URL
+                         redirect.toString(QUrl.FormattingOption(QUrl.RemoveAuthority | QUrl.RemoveScheme)),
+                         handler, xml, priority=True, important=True)
+            elif xml:
                 xml_handler = XmlHandler()
                 xml_handler.init()
                 xml_reader = QtXml.QXmlSimpleReader()
@@ -209,10 +260,10 @@ class XmlWebService(QtCore.QObject):
             queue = self._high_priority_queues.get(key) or self._low_priority_queues.get(key)
             if not queue:
                 continue
-            now = QtCore.QTime.currentTime()
+            now = time.time()
             last = self._last_request_times.get(key)
             request_delay = REQUEST_DELAY[key]
-            last_ms = last.msecsTo(now) if last is not None else request_delay
+            last_ms = (now - last) * 1000 if last is not None else request_delay
             if last_ms >= request_delay:
                 self.log.debug("Last request to %s was %d ms ago, starting another one", key, last_ms)
                 d = request_delay
@@ -260,10 +311,6 @@ class XmlWebService(QtCore.QObject):
         if params: path += "&" + "&".join(params)
         return self.get(host, port, path, handler, priority=priority, important=important, mblogin=mblogin)
 
-    def get_release_group_by_id(self, releasegroupid, handler, priority=True, important=True):
-        inc = ['releases', 'media']
-        return self._get_by_id('release-group', releasegroupid, handler, inc, priority=priority, important=important)
-
     def get_release_by_id(self, releaseid, handler, inc=[], priority=True, important=False, mblogin=False):
         return self._get_by_id('release', releaseid, handler, inc, priority=priority, important=important, mblogin=mblogin)
 
@@ -292,7 +339,7 @@ class XmlWebService(QtCore.QObject):
         if query: filters.append(('query', ' '.join(query)))
         params = []
         for name, value in filters:
-            value = str(QtCore.QUrl.toPercentEncoding(QtCore.QString(value)))
+            value = str(QUrl.toPercentEncoding(QtCore.QString(value)))
             params.append('%s=%s' % (str(name), value))
         path = "/ws/2/%s/?%s" % (entitytype, "&".join(params))
         return self.get(host, port, path, handler)
@@ -303,6 +350,17 @@ class XmlWebService(QtCore.QObject):
     def find_tracks(self, handler, **kwargs):
         return self._find('recording', handler, kwargs)
 
+    def _browse(self, entitytype, handler, kwargs, inc=[], priority=False, important=False):
+        host = self.config.setting["server_host"]
+        port = self.config.setting["server_port"]
+        params = "&".join(["%s=%s" % (k, v) for k, v in kwargs.items()])
+        path = "/ws/2/%s?%s&inc=%s" % (entitytype, params, "+".join(inc))
+        return self.get(host, port, path, handler, priority=priority, important=important)
+
+    def browse_releases(self, handler, priority=True, important=True, **kwargs):
+        inc = ["media", "labels"]
+        return self._browse("release", handler, kwargs, inc, priority=priority, important=important)
+
     def submit_puids(self, puids, handler):
         path = '/ws/2/recording/?client=' + USER_AGENT_STRING
         recordings = ''.join(['<recording id="%s"><puid-list><puid id="%s"/></puid-list></recording>' % i for i in puids.items()])
@@ -322,10 +380,37 @@ class XmlWebService(QtCore.QObject):
         host, port = 'ofa.musicdns.org', 80
         filters = []
         for name, value in kwargs.items():
-            value = str(QtCore.QUrl.toPercentEncoding(value))
+            value = str(QUrl.toPercentEncoding(value))
             filters.append('%s=%s' % (str(name), value))
         return self.post(host, port, '/ofa/1/track/', '&'.join(filters), handler, mblogin=False)
 
+    def _encode_acoustid_args(self, args):
+        filters = []
+        args['client'] = ACOUSTID_KEY
+        args['clientversion'] = version_string
+        args['format'] = 'xml'
+        for name, value in args.items():
+            value = str(QUrl.toPercentEncoding(value))
+            filters.append('%s=%s' % (str(name), value))
+        return '&'.join(filters)
+
+    def query_acoustid(self, handler, **args):
+        host, port = 'api.acoustid.org', 80
+        body = self._encode_acoustid_args(args)
+        return self.post(host, port, '/v2/lookup', body, handler, mblogin=False)
+
+    def submit_acoustid_fingerprints(self, submissions, handler):
+        args = {'user': self.config.setting["acoustid_apikey"]}
+        for i, submission in enumerate(submissions):
+            args['fingerprint.%d' % i] = str(submission.fingerprint)
+            args['duration.%d' % i] = str(submission.duration)
+            args['mbid.%d' % i] = str(submission.trackid)
+            if submission.puid:
+                args['puid.%d' % i] = str(submission.puid)
+        host, port = 'api.acoustid.org', 80
+        body = self._encode_acoustid_args(args)
+        return self.post(host, port, '/v2/submit', body, handler, mblogin=False)
+
     def download(self, host, port, path, handler, priority=False, important=False):
         return self.get(host, port, path, handler, xml=False, priority=priority, important=important)
 
